/* IODEV.C                                                                   */

/* #define TRC_IO 0 */


/*---------------------------------------------------------------------------*/
/* Write output data to port                                                 */
/* If port is not readable, save the value for simulated readbacks           */

IOWritePort(DevNdx,DevOffs,Value)
WORD DevNdx;
WORD DevOffs;
WORD Value;
{

   if (REPORT_DEV & ReportMode) {
      sprintf(CmdRespBuff,"Write %02x to address %04x\n",
              Value,IOAddress[DevNdx]+DevOffs);
      CmdShowMsg(CmdRespBuff);
   }

   poke(IOAddress[DevNdx]+DevOffs,Value);

   if (IO_WRITEONLY & IOFlags[DevNdx][DevOffs]) {
      IOCurrent[DevNdx][DevOffs] = Value;
   }

}


/*---------------------------------------------------------------------------*/
/* Read data from port                                                       */
/* If port is not readable, we use the value stored in the IOCurrent array   */

WORD IOReadPort(DevNdx,DevOffs)
WORD DevNdx;
WORD DevOffs;
{
WORD RetValue;
BYTE Faked;


   if (IO_WRITEONLY & IOFlags[DevNdx][DevOffs]) {
      RetValue = IOCurrent[DevNdx][DevOffs];
      Faked = 1;
   }
   else {
      RetValue = peek(IOAddress[DevNdx]+DevOffs);
      Faked = 0;
   }

   if (REPORT_DEV & ReportMode) {
      sprintf(CmdRespBuff,"Read %02x from address %04x %s\n",
              RetValue,IOAddress[DevNdx]+DevOffs,
              Faked ? "faked" : "");
      Nputstr(CmdRespBuff);
   }

   return RetValue;

}


/*---------------------------------------------------------------------------*/
/* Set up I/O devices and initialize the arrays                              */

IOInitialize() {

BYTE IODevNdx;

/*--- set interrupt vector (this is not really executed on this call...)     */

 asm {
EX0HERE EQU *
   ORG   $0003                   /* External interrupt 0 (clock tick)        */
   LJMP  Int0Tick
   ORG   EX0HERE
 }

/*--- global variables                                                       */

   Sampling = 0;                       /* shut down automatic sampling       */
   Radix = 16;                         /* start out in HEX mode              */

   CmdShowMsg("Initializing device information\n");

   memset(IOFlags,0,sizeof(IOFlags));

   memset(pIOBuffer,0,sizeof(pIOBuffer));
   memset(IOBuffLength,0,sizeof(IOBuffLength));
   memset(IOBuffCount,0,sizeof(IOBuffCount));

   memset(IOCurrent,0,sizeof(IOCurrent));
   memset(IOBuffPeriod,0,sizeof(IOBuffPeriod));
   memset(IOBuffTimer,0,sizeof(IOBuffTimer));

/*--- now check out each device to see if it is present                      */

   for (IODevNdx=0; IODevNdx<NUMDEVS; ++IODevNdx) {

      sprintf(CmdRespBuff," ...%s: ",IOName[IODevNdx]);
      CmdShowMsg(CmdRespBuff);

      switch (IODevNdx) {
      case DEV_AI0 :
      case DEV_AI1 :
         InstADC(IODevNdx);
         break;
      case DEV_AO0 :
      case DEV_AO1 :
         InstDAC(IODevNdx);
         break;
      case DEV_CLK0 :
      case DEV_CLK1 :
         InstCLK(IODevNdx);
         break;
      case DEV_DP0 :
      case DEV_DP1 :
         InstI55(IODevNdx);
         break;
      default :
         CmdShowMsg("assume not installed");
      }
      CmdShowMsg("\n");
   }


/*--- do hardware setups that depend on the I/O devices                      */

   if (IO_INSTALLED & IOFlags[DEV_CLK0][0]) {
      SETBIT(IT0);               /* interrupt on falling edge only           */
      SETBIT(EX0);               /* enable time of day sync interrupt        */
      SETBIT(FlagFracts);        /* turn on fractional seconds               */
   }
   else {
      CLRBIT(IT0);
      CLRBIT(EX0);
   }


}


/*---------------------------------------------------------------------------*/
/* Initialization routines                                                   */
/* Returns 0 if present & installed, nonzero otherwise                       */

/*--- Analog to digital converters                                           */
/*    If AI0, we check to see if EOC signal comes in on INT1                 */
/*      and then use the timing to decide which ADC we've got                */
/*    If AI1, we default to 0808 and hope for the best                       */
/*    The hardware inverts the EOC signal, so LOW = DONE                     */

int  InstADC(DevNdx)
WORD DevNdx;
{
int RetVal;
BYTE Loops;

   RetVal = 0;

   TmrDelayTicks(2);                   /* ensure ANY conversion is finished  */

   switch (DevNdx) {
   case DEV_AI0 :
      Loops = 0;
      poke(IOAddress[DevNdx],7);                   /* start the conversion   */
 asm {
      CLR   EA                         ; ensure no interruptions
      MOV   B,#0                       ; take 256 samples
      CLR   A                          ; count samples while converting
?lp2
      MOV   C,ADC_EOC                  ; HIGH while converting (or dead)
      ADDC  A,#0
      DJNZ  B,?lp2                     ; about 5 us per loop

      SETB  EA
      MOV   TestBits,A                 ; stash result

 }
      if (TestBits) {
         IOFlags[DevNdx][0] |= IO_INSTALLED;       /* assume installed       */
         strcpy(CmdRespBuff,"type is ");
         if (9 > TestBits) {
            IOFlags[DevNdx][0] |= IO_ALTDEV | IO_WORD;        /* 0810...     */
            strcat(CmdRespBuff,"SDA0810");
         }
         else {
            strcat(CmdRespBuff,"ADC0808");
         }
      }
      else {
         strcpy(CmdRespBuff,"not installed");
         RetVal = 1;
      }
      CmdShowMsg(CmdRespBuff);
      sprintf(CmdRespBuff,", EOC tests=%u",TestBits);
      CmdShowMsg(CmdRespBuff);

      break;
   case DEV_AI1:
      IOFlags[DevNdx][0] |= IO_INSTALLED;
      CmdShowMsg("assume ADC0808");
      break;
   default :
   }

   return RetVal;
}


/*--- Digital to analog converters                                           */
/*    cannot read back the outputs, so we set them to zeros and be done      */

int  InstDAC(DevNdx)
WORD DevNdx;
{
int RetVal;
BYTE Channel;

   RetVal = 0;

   for (Channel=0; Channel<IONumChan[DevNdx]; ++Channel) {
      IOFlags[DevNdx][Channel] |= IO_WRITEONLY;
   }

   switch (DevNdx) {
   case DEV_AO0 :
   case DEV_AO1 :
      IOFlags[DevNdx][0] |= IO_INSTALLED;
      CmdShowMsg("assume installed, set outputs to 00");
      for (Channel=0; Channel<IONumChan[DevNdx]; ++Channel) {
         IOWritePort(DevNdx,Channel,0);
      }
      break;
   }


   return RetVal;

}

/*--- Real time clock                                                        */
/*    CLK0: verify that the chip is present and set up 1 second interrupts   */
/*    For CLK1, verify presence and disable interrupts                       */

int  InstCLK(DevNdx)
WORD DevNdx;
{
int RetVal;
BYTE ClkReg;

   RetVal = 0;

/*--- see if we can set and read back a register                             */

   RetVal = 0;                                     /* assume present         */

   poke(IOAddress[DevNdx]+0x0D,0x07);              /* IRQ | BUSY | HOLD      */
   ClkReg = peek(IOAddress[DevNdx]);               /* save unit seconds      */

   poke(IOAddress[DevNdx],6);
   if (6 != (0x0F & peek(IOAddress[DevNdx]))) {
      RetVal = 1;
   }
   poke(IOAddress[DevNdx],9);
   if (9 != (0x0F & peek(IOAddress[DevNdx]))) {
      RetVal = 1;
   }

   poke(IOAddress[DevNdx],ClkReg);                 /* restore seconds        */
   poke(IOAddress[DevNdx]+0x0D,0x06);              /* clear HOLD flag        */

   if (RetVal) {                                   /* bail out if missing    */
      CmdShowMsg("not installed");
      goto Done;
   }
   else {
      CmdShowMsg("installed");
   }

   IOFlags[DevNdx][0] |= IO_INSTALLED;             /* yes!  got hardware!    */

/*--- set up registers appropriately                                         */

   switch (DevNdx) {
   case DEV_CLK0 :
      poke(IOAddress[DevNdx]+0x0E,0x04);           /* 1 sec | Stnd  | ~Mask  */
      CmdShowMsg(", int enabled");
      ClkReg = 0x0F & peek(IOAddress[DevNdx]+0x0F);/* check 24 hr setup      */
      if ((!(0x04 & ClkReg)) || (0x01 & ClkReg)) {
         poke(IOAddress[DevNdx]+0x0F,0x01);        /* Reset                  */
         poke(IOAddress[DevNdx]+0x0F,0x05);        /* 24 hr mode             */
         poke(IOAddress[DevNdx]+0x0F,0x04);        /* 24 hr w/o reset        */
         CmdShowMsg(", 24 hr mode, started");
      }
      break;
   case DEV_CLK1:
      poke(IOAddress[DevNdx]+0x0E,0x01);           /* mask off interrupts    */
      CmdShowMsg(", int disabled, not set up");
      break;
   default :
   }


Done:

   return RetVal;

}

/*--- Intel 8255 programmable I/O chip                                       */
/*    This gets set up with Port C as output to match ADIO buffer hardware   */

int  InstI55(DevNdx)
WORD DevNdx;
{
int RetVal;

   RetVal = 0;

   IOFlags[DevNdx][0] |= IO_INSTALLED;             /* applies to all regs    */
   IOFlags[DevNdx][2] |= IO_OUTPUT;                /* this port is an output */
   IOFlags[DevNdx][3] |= IO_WRITEONLY;             /* cannot read ctl ports  */

   IOWritePort(DevNdx,3,0x92);                     /* force mode 0           */

   CmdShowMsg("assume installed, channels 0-1 = input, 2 = output");

   return RetVal;

}


/*---------------------------------------------------------------------------*/
/* External Interrupt 0                                                      */
/* This should be the one-second tick from the real time clock chip          */
/* If it's not installed, we don't get any ticks!                            */

Int0Tick() {

;
 asm {
   PUSH  A
   PUSH  PSW

*--- force the 8031 internal timer to sync with external timer

   CLR   TR0
   MOV   TL0,T0Reload
   MOV   TH0,T0Reload+1
   SETB  TR0

*--- adjust fractional seconds value
*    the timer interrupt makes FracSec match this value

   MOV   A,T0Ticks
   JZ    ?I0Done                       ; zero means we just ticked, so OK
   CJNE  A,#TICKSPERSEC-1,?flush       ; if not about to tick, flush fraction

   SETB  TF0                           ; force interrupt if about to tick
   SJMP  ?I0Done

?flush   EQU   *
   MOV   T0Ticks,#0
   MOV   FracSec,#0

*--- restore regs and return...

?I0Done  EQU   *

   POP   PSW
   POP   A

   RETI                          ; pre-empt normal C level return

 }

}


/*---------------------------------------------------------------------------*/
/* Convert ADC input                                                         */
/* Control is ADC_NORMAL to use EOC input, ADC_TIMED to delay a fixed time   */
/* Returns the converted value, FFFF if there is a problem we can detect     */
/* (Actually, for now we just use the timer to delay about 100 us...)        */

#define ADC_TIMEONLY

WORD IOConvADC(DevNdx,DevChan,Ctl)
WORD DevNdx;
WORD DevChan;
int Ctl;
{
WORD RetVal;

#ifndef ADC_TIMEONLY
int Deadman;
#endif

/*--- if EOC input is not DONE (low), we force timed mode                    */

   if (ADC_NORMAL == Ctl) {
      TESTBIT(TestBits,ADC_EOC);
      if (TestBits) {
         Ctl = ADC_TIMED;
         if (TestBits && (REPORT_ADC & ReportMode)) {
            sprintf(CmdRespBuff,"*** %s EOC inactive before convert\n",
                     IOName[DevNdx]);
            CmdShowMsg(CmdRespBuff);
         }
      }
   }

/*--- start the conversion and force delay to ensure EOC goes active         */
/*    max delay is about 20 microseconds                                     */

   poke(IOAddress[DevNdx]+DevChan,0);              /* valid is not used      */

#ifndef ADC_TIMEONLY

 asm {
      MOV   B,#10                      ; at 2 * 1.085 microseconds per loop
?here DJNZ  B,?here
 }

#else

 asm {
      MOV   B,#70                      ; about 140 microseconds...
?here DJNZ  B,?here                    ; at 2 * 1.085 us/loop
 }

#endif

/*--- if using EOC, wait for it to pop low again                             */
/*    otherwise, just delay for the maximum possible conversion time...      */

#ifndef ADC_TIMEONLY

   Deadman = 0;
   if (ADC_NORMAL == Ctl) {
      do {
         ++Deadman;
         TESTBIT(TestBits,ADC_EOC);
      } while (TestBits && (Deadman < 100));
      if (TestBits && (REPORT_ADC & ReportMode)) {
         sprintf(CmdRespBuff,"*** %s timeout after %u iterations\n",
                  IOName[DevNdx],Deadman);
         CmdShowMsg(CmdRespBuff);
      }
   }
   else {
      TmrDelay100US((IO_ALTDEV & IOFlags[DevNdx][0]) ? 1 : 3);
      TestBits = 0;
   }

#else
   TestBits = 0;                       /* fake no timeout...                 */
#endif

/*--- now read the results                                                   */

   if (TestBits) {                     /* if there was a timeout             */
      RetVal = 0xFFFF;
   }
   else {
      if (IO_ALTDEV & IOFlags[DevNdx][0]) {
         RetVal =  (0x00FF & (WORD)peek(IOAddress[DevNdx])) << 2;
         RetVal |= (0x00FF & (WORD)peek(IOAddress[DevNdx])) >> 6;
      }
      else {
         RetVal = (WORD)peek(IOAddress[DevNdx]);            /* LSB only      */
      }
   }

   if (TestBits && (REPORT_ADC & ReportMode)) {
      sprintf(CmdRespBuff," %s.%u = %02x\n",IOName[DevNdx],DevChan,RetVal);
      CmdShowMsg(CmdRespBuff);
   }

   return RetVal;

}



/*---------------------------------------------------------------------------*/
/* Parse I/O device specifier into array index, channel offset, and bit ID   */
/* Argument is pointer to pointer to characters                              */
/* Returns index in high byte, channel in low byte                           */
/* Returns NODEVSPEC if device is no good                                    */
/* Always returns char ptr incremented past last char used                   */

int IOParseDev(ppSpec)
char **ppSpec;
{
WORD RetVal;
char *pBuffer;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
BYTE Temp;

   DevChan = DevBitID = 0;
   RetVal = NODEVSPEC | NOCHANSPEC | NOBITSPEC;    /* assume the worst       */

/*--- extract device string up to first non-alphanumeric character           */
/*    then look it up in the table for a match                               */

   memset(IOParseBuff,0,sizeof(IOParseBuff));
   pBuffer = IOParseBuff;
   while (isascii(**ppSpec) && isalnum(**ppSpec)) {         /* force caps    */
      *(pBuffer++) = toupper(**ppSpec);
      ++(*ppSpec);
   }

   for (DevNdx=0; DevNdx<NUMDEVS; ++DevNdx) {
      if (!strcmp(IOParseBuff,IOName[DevNdx])) {
         break;
      }
   }

   if (NUMDEVS == DevNdx) {
      sprintf(CmdRespBuff,"*** Bad device name [%s]\n",IOParseBuff);
      CmdShowMsg(CmdRespBuff);
      goto Done;
   }
   else {
      RetVal &= ~NODEVSPEC;
      if (REPORT_DEV & ReportMode) {
         sprintf(CmdRespBuff,"[Device %s",IOParseBuff);
         Nputstr(CmdRespBuff);
      }
   }


/*--- extract channel spec if we ended at a dot                              */

   if ('.' == **ppSpec) {
      ++(*ppSpec);
      Temp = GetDecimal(ppSpec);
      if (Temp < IONumChan[DevNdx]) {
         DevChan = Temp;
         RetVal &= ~NOCHANSPEC;
         if (REPORT_DEV & ReportMode) {
            sprintf(CmdRespBuff,", channel = %u",Temp);
            Nputstr(CmdRespBuff);
         }
      }
      else {
         RetVal = NODEVSPEC;
         sprintf(CmdRespBuff,"\n*** Bad channel for %s: %u\n",
                  IOName[DevNdx],Temp);
         CmdShowMsg(CmdRespBuff);
         goto Done;
      }
   }
   else {
      if (REPORT_DEV & ReportMode) {
         Nputstr(", all channels");
      }
   }

/*--- extract bit spec if we are once again at a dot                         */

   if ('.' == **ppSpec) {
      ++(*ppSpec);
      Temp = GetDecimal(ppSpec);
      if (((Temp < 16) && (IO_WORD & IOFlags[DevNdx][0])) ||
          ((Temp <  8) && !(IO_WORD & IOFlags[DevNdx][0]))) {
         RetVal &= ~NOBITSPEC;
         DevBitID = Temp;
         if (REPORT_DEV & ReportMode) {
            sprintf(CmdRespBuff,", bit ID = %u]\n",Temp);
            Nputstr(CmdRespBuff);
         }
      }
      else {
         RetVal = NODEVSPEC;
         sprintf(CmdRespBuff,"\n*** Bad bit ID: %u\n",Temp);
         CmdShowMsg(CmdRespBuff);
         goto Done;
      }
   }
   else {
      if (REPORT_DEV & ReportMode) {
         Nputstr("]\n");
      }
   }

/*--- combine the parts and return a single word                             */
/*    preserves all the flags in the high nibble                             */
/*     (values will be zero if flags are set, but so it goes                 */

Done:

   RetVal |= (DevNdx << 8) | (DevChan << 4) | DevBitID;

   return RetVal;

}



/*---------------------------------------------------------------------------*/
/* Extract I/O device spec and value from string                             */
/* Returns string pointer ticked to just beyond the value... if any          */
/* Extracts device index/channel/bit and returns the code                    */
/* Function return value is 0 for OK, 1 for error                            */
/* Handles clock timestamps as special cases                                 */

int IOSetValue(ppCmd,pDevCode)
char **ppCmd;
WORD *pDevCode;
{
int RetCode;
WORD DevCode;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
WORD Value;
WORD PortVal;

   RetCode = 1;                                    /* assume failure         */

   *pDevCode = DevCode = IOParseDev(ppCmd);        /* get device & channel   */
   if (NODEVSPEC & DevCode) {                      /* skip invalid devs      */
      goto Done;
   }

   DevNdx =   IO_DEVICE(DevCode);
   DevChan =  IO_CHANNEL(DevCode);
   DevBitID = IO_BITID(DevCode);

   if (('=' != **ppCmd) || !(*((*ppCmd)+1))) {     /* check for =string      */
      CmdShowMsg("*** Set command missing data\n");
      goto Done;
   }

   ++(*ppCmd);                                     /* step over = sign       */

   if (NOCHANSPEC & DevCode) {                     /* CLK has special case   */
      switch (DevNdx) {
      case DEV_CLK0 :
      case DEV_CLK1 :
         RetCode = IOSetTime(ppCmd,DevCode);
         break;
      default :
         CmdShowMsg("*** Missing channel\n");
      }
   }
   else {                                          /* ordinary I/O string    */
      switch (Radix) {
      case 16 :
      default :
         Value = GetHexits(ppCmd,4);               /* get hex word if there  */
         break;
      case 10 :
         Value = GetDecimal(ppCmd);                /* get decimal            */
         break;
      }

      if (NOBITSPEC & DevCode) {                   /* missing bit spec?      */
         IOWritePort(DevNdx,DevChan,Value);        /* write whole byte       */
      }
      else {                                       /* have bit map spec      */
         if (Value > 1) {                          /* is data OK?            */
            CmdShowMsg("*** Bit value must be 0 or 1\n");
            goto Done;
         }
         PortVal = IOReadPort(DevNdx,DevChan);  /* fetch existing data    */
         PortVal &= ~(0x01 << DevBitID);        /* remove old bit         */
         PortVal |= Value << DevBitID;          /* insert new bit         */
         IOWritePort(DevNdx,DevChan,PortVal);   /* write whole byte       */
      }
      RetCode = 0;
   }


Done:

   return RetCode;

}



/*---------------------------------------------------------------------------*/
/* Extract I/O device from incoming string                                   */
/* Returns value from appropriate port or bit                                */
/* Function return value is 0 for OK, 1 for error                            */
/* Returns string pointer ticked to just beyond the value... if any          */
/* Returns decoded value of device and channel                               */
/* If the device is not installed, displays error message                    */
/* This handles the clocks as a special case, returning zero with no         */
/*  channel specified...                                                     */

int IOGetValue(ppCmd,pDevCode,pValue)
char **ppCmd;
WORD *pDevCode;
WORD *pValue;
{

int RetCode;
WORD DevCode;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
WORD Value;

   *pDevCode = DevCode = IOParseDev(ppCmd);        /* get device & channel   */
   RetCode = 1;                                    /* assume error           */
   Value = 0;                                      /* just in case...        */

   if (NODEVSPEC & DevCode) {                      /* bad device             */
      goto Done;
   }

   DevNdx =   IO_DEVICE(DevCode);
   DevChan =  IO_CHANNEL(DevCode);
   DevBitID = IO_BITID(DevCode);

   if (!(IO_INSTALLED & IOFlags[DevNdx][0])) {              /* installed?    */
      sprintf(CmdRespBuff,"*** Device %s not installed\n",IOName[DevNdx]);
      CmdShowMsg(CmdRespBuff);
      goto Done;
   }

   if ('=' == **ppCmd) {                           /* check for = sign!      */
      CmdShowMsg("*** Cannot set value\n");
      goto Done;
   }
   else {

      if (NOCHANSPEC & DevCode) {                  /* missing channel?       */
         switch (DevNdx) {
         case DEV_CLK0 :
         case DEV_CLK1 :
            RetCode = 0;                           /* OK & bypass for these  */
            Value = 0;
            goto Done;
         default :
            DevChan = 0;                           /* default to zero here   */
         }
      }

      if (NOBITSPEC & DevCode) {
         switch (DevNdx) {                         /* no bit, fetch byte     */
         case DEV_AI0 :
         case DEV_AI1 :
            Value = IOConvADC(DevNdx,DevChan,
      (IO_INSTALLED & IOFlags[DevNdx][DevChan]) ? ADC_NORMAL : ADC_TIMED);
            break;
         default :
            Value = IOReadPort(DevNdx,DevChan);
         }
      }
      else {                                       /* use bit spec on data   */
         Value = IOReadPort(DevNdx,DevChan);       /* fetch existing data    */
         Value >>= DevBitID;                       /* move bit to LSB        */
         Value &= 0x0001;                          /* isolate it             */
      }
   }


   RetCode = 0;

Done:

   *pValue = Value;

#ifdef TRC_IO
   sprintf(CmdRespBuff,"Getval DevCode @%04x=%04x, Value @%04x=%02x\n",
            pDevCode,DevCode,pValue,Value);
   Nputstr(CmdRespBuff);
#endif

   return RetCode;

}



/*---------------------------------------------------------------------------*/
/* Formats value for given device and channel                                */
/* Produces appropriate text to identify the results                         */
/* Message for non-installed devices                                         */

int IOFormatData(DevCode,Value)
WORD DevCode;
WORD Value;
{
int RetCode;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
char *pFormat;

   RetCode = 1;

#ifdef TRC_IO
   sprintf(CmdRespBuff,"<IOFormatData DevCode %04x, Value %02x>",
            DevCode,Value);
   Nputstr(CmdRespBuff);
#endif

   if (NODEVSPEC & DevCode) {
      strcpy(CmdRespBuff,"*** Bad device\n");
      goto Done;
   }

   DevNdx =   IO_DEVICE(DevCode);
   DevChan =  IO_CHANNEL(DevCode);
   DevBitID = IO_BITID(DevCode);

   strcpy(CmdRespBuff,IOName[DevNdx]);             /* set up device name     */

   if (!(NOCHANSPEC & DevCode)) {
      sprintf(CmdRespBuff+strlen(CmdRespBuff),".%u",DevChan);
      if (!(NOBITSPEC & DevCode)) {
         sprintf(CmdRespBuff+strlen(CmdRespBuff),".%u",DevBitID);
      }
   }

   strcat(CmdRespBuff,"=");

   if (!(NOBITSPEC & DevCode)) {                   /* have bit specifier     */
      pFormat = "%u";                              /* either 0 or 1          */
   }
   else {
      switch (Radix) {
      case 16 :
      default :
         pFormat = (IO_WORD & IOFlags[DevNdx][0]) ? "%04x" : "%02x";
         break;
      case 10 :
         pFormat = "%u";
         break;
      }
   }

   sprintf(CmdRespBuff+strlen(CmdRespBuff),pFormat,Value);

   RetCode = 0;

Done:

   if (RetCode) {
      CmdShowMsg(CmdRespBuff);                     /* suppressable errors    */
   }
   else {
      Nputstr(CmdRespBuff);                        /* normal results         */
   }

   return RetCode;

}

/*---------------------------------------------------------------------------*/
/* Un-format time string and set the clock                                   */
/* Uses IOTimeFormat to parse the string back to register values             */
/* Fractional seconds are automatically cleared when next second ticks       */
/* Function return value is 0 for OK, 1 for error                            */

int IOSetTime(ppCmd,DevCode)
char **ppCmd;
WORD DevCode;
{
int RetCode;
unsigned char *pFormat;
char *pChar;
BYTE Reg;

   RetCode = 1;

   pFormat = IOTimeFormat;                         /* aim at format string   */

   while ((**ppCmd) && (',' != **ppCmd) && (*pFormat)) {

#ifdef TRC_IO
   sprintf(CmdRespBuff,"<string [%s], format %02x>\n",
            *ppCmd,*pFormat);
   Nputstr(CmdRespBuff);
#endif

      if (' ' == **ppCmd) {                        /* strip blanks           */
         ++(*ppCmd);
         continue;
      }

      if (' ' == *pFormat) {                       /* from format, too       */
         ++pFormat;
         continue;
      }

      if (0x80 > *pFormat) {
         if ((*pFormat) != **ppCmd) {
            sprintf(CmdRespBuff,
                    "**** Separator mismatch at [%s], clock may be bad\n",
                    *ppCmd);
            CmdShowMsg(CmdRespBuff);
            goto Done;
         }
         ++pFormat;                                /* skip literal char      */
         ++(*ppCmd);                               /* both places            */
      }
      else {
         switch (*pFormat) {
         case 0x90 :                               /* fractional seconds     */
            while (isascii(**ppCmd) && isdigit(**ppCmd)) {  /* skip digits   */
               ++(*ppCmd);
            }
            break;
         case 0x91 :                               /* day of week            */
            pChar = IOClkBuff;                     /* extract to buffer      */
            memset(IOClkBuff,0,sizeof(IOClkBuff));
            while (isascii(**ppCmd) && isalpha(**ppCmd)) {
               *pChar = toupper(**ppCmd);
               ++pChar;
               ++(*ppCmd);
            }
            for (Reg=0; Reg<7; ++Reg) {            /* look up in table       */
               if (!strcmp(IOClkBuff,IODays[Reg])) {
                  IOWritePort(IO_DEVICE(DevCode),12,Reg);
                  break;
               }
            }
            if (8 == Reg) {
               sprintf(CmdRespBuff,
                       "**** Invalid day of week [%s], clock may be bad\n",
                       IOClkBuff);
               CmdShowMsg(CmdRespBuff);
               goto Done;
            }
            break;
         default :                                 /* register formatters    */
            if (isascii(**ppCmd) && isdigit(**ppCmd)) {
               Reg = **ppCmd - '0';                /* convert to digit       */
               IOWritePort(IO_DEVICE(DevCode),(*pFormat)-0x80,Reg);
               ++(*ppCmd);
            }
            else {
               sprintf(CmdRespBuff,
                       "**** Invalid time string at [%s], clock may be bad\n",
                       *ppCmd);
               CmdShowMsg(CmdRespBuff);
               goto Done;
            }
         }
         ++pFormat;
      }

   }


Done:
   return RetCode;


}

/*---------------------------------------------------------------------------*/
/* Puts formatted clock timestamp string into IOClkBuff                      */
/* Halts the clock while reading to avoid bad results                        */
/* Returns 0 for OK, 1 for error                                             */

int IOFormatClock(DevNdx,pFormat)
WORD DevNdx;
unsigned char *pFormat;
{
int RetCode;
char *pOut;
BYTE Reg;
BYTE Busy;
BYTE WasBusy;

   RetCode = 1;

   if (!(IO_INSTALLED & IOFlags[DevNdx][0])) {
      *IOClkBuff = 0;
      goto Done;
   }

   do {
      IOWritePort(DevNdx,13,0x5);                  /* turn HOLD on, IRQ=1    */
      Busy = 0x02 & IOReadPort(DevNdx,13);         /* fetch BUSY flag        */
      if (Busy) {                                  /* if BUSY active...      */
         WasBusy = 1;                              /* remember for later     */
         IOWritePort(DevNdx,13,0x4);               /*  ... turn HOLD off     */
      }
   } while (Busy);                                 /* until no longer BUSY   */

   pOut = IOClkBuff + strlen(IOClkBuff);
   while (*pFormat) {
      if (0x80 <= *pFormat) {
         switch (*pFormat) {
         case 0x90 :
            sprintf(pOut,"%03u",MSPERTICK*(WORD)FracSec);
            pOut += 2;
            break;
         case 0x91 :
            Reg = 0x0F & IOReadPort(DevNdx,12);
            strcpy(pOut,IODays[Reg]);
            pOut += strlen(IODays[Reg]) - 1;
            break;
         default :
            *pOut = '0' + (0x0F & IOReadPort(DevNdx,(*pFormat)-0x80));
         }
      }
      else {
         *pOut = *pFormat;
      }
      ++pOut;
      ++pFormat;
   }

   IOWritePort(DevNdx,13,0x4);                     /* turn HOLD off...       */
#ifdef x0
   strcat(IOClkBuff,WasBusy ? " *" : "  ");        /* indicate if hit busy   */
#endif

   RetCode = 0;

Done:

   return;

}

/*---------------------------------------------------------------------------*/
/* Outputs formatted clock timestamp with header                             */
/* This is independent of the Radix setting, for obvious reasons...          */
/* Halts the clock while reading to avoid bad results                        */
/* Returns 0 for OK, 1 for error                                             */

int IOFormatTime(DevCode,pFormat)
WORD DevCode;
unsigned char *pFormat;
{
int RetCode;
BYTE DevNdx;
BYTE DevChan;

   RetCode = 1;

   if (NODEVSPEC & DevCode) {
      strcpy(IOClkBuff,"*** Cannot show time, bad device code\n");
      goto Done;
   }

   DevNdx = IO_DEVICE(DevCode);
   DevChan = IO_CHANNEL(DevCode);

   if (!(IO_INSTALLED & IOFlags[DevNdx][0])) {
      sprintf(IOClkBuff,"*** Device %s not installed\n",IOName[DevNdx]);
      goto Done;
   }

   strcpy(IOClkBuff,IOName[DevNdx]);               /* set up device name     */
   strcat(IOClkBuff,"=");

   RetCode = IOFormatClock(DevNdx,pFormat);

Done:

   if (RetCode) {
      CmdShowMsg(IOClkBuff);                       /* can suppress errors    */
   }
   else {
      Nputstr(IOClkBuff);                          /* show the results       */
   }

   return;

}

/*---------------------------------------------------------------------------*/
/* Dump all of the devices for a given board                                 */
/* The AI and AO devices use two-byte data regardless of the actual device   */
/* Make sure the IODumpFormat string matches the default time string below   */
/*  so you produce the same number of characters in either case              */

IODumpAll(BoardID)
int BoardID;
{
WORD DevNdx;
WORD DevChan;
unsigned char *pFormat;

   sprintf(CmdRespBuff,"QA%d=",BoardID);
   Nputstr(CmdRespBuff);

   DevNdx = BoardID ? DEV_AI1 : DEV_AI0;
   for (DevChan=0; DevChan<8; ++DevChan) {
      sprintf(CmdRespBuff,"%04x ",IOConvADC(DevNdx,DevChan,ADC_NORMAL));
      Nputstr(CmdRespBuff);
   }

   DevNdx = BoardID ? DEV_AO1 : DEV_AO0;
   for (DevChan=0; DevChan<4; ++DevChan) {
      sprintf(CmdRespBuff,"%04x ",IOReadPort(DevNdx,DevChan));
      Nputstr(CmdRespBuff);
   }

   DevNdx = BoardID ? DEV_DP1 : DEV_DP0;
   for (DevChan=0; DevChan<4; ++DevChan) {
      sprintf(CmdRespBuff,"%02x ",IOReadPort(DevNdx,DevChan));
      Nputstr(CmdRespBuff);
   }

   pFormat = IODumpFormat;
   DevNdx = BoardID ? DEV_CLK1 : DEV_CLK0;
   *IOClkBuff = 0;
   if (IO_INSTALLED & IOFlags[DevNdx][0]) {
      IOFormatClock(DevNdx,pFormat);
   }
   else {
      strcpy(IOClkBuff,"000000 000000000");
   }
   Nputstr(IOClkBuff);                             /* show the results       */

}
