/* EECONFIG.C                                                                */

#define TRC_CFG


/*---------------------------------------------------------------------------*/
/* Dump configuration data and so forth                                      */

CfgDumpState() {

BYTE Index;

   sprintf("EEPStatus: $02x\n",EEPStatus);

   Nputstr("EEPROM contents:\n");
   for (Index=0; Index < (EEPROMSIZE/4); ++Index) {
      sprintf(CmdRespBuff," %02x:%04x  %02x:%04x  %02x:%04x  %02x:%04x\n",
         Index,    EEImage[Index],
         Index+16, EEImage[Index+16],
         Index+32, EEImage[Index+32],
         Index+48, EEImage[Index+48]);
      CmdShowMsg(CmdRespBuff);
      TmrHeartBeat();
   }





}

/*---------------------------------------------------------------------------*/
/* Send command and data to EEPROM                                           */
/* Returns data for read command,                                            */
/*  0 if write command was successful, 1 if it timed out                     */
/* CAUTION -- the EEPROM will jam if you write to an un-erased entry!        */
/* Remember that the heartbeat output is also the EEPROM enable control...   */
/*   so we must manually update the heartbeat flag when we flip the output   */

WORD CfgSendEECmd(CmdID,Data)
WORD CmdID;
WORD Data;
{

   TmrHeartBeat();                     /* get up to date                     */

/*--- send out the header, command, and address bits                         */
/*    if the EEPROM is jammed due to a write with no erase, we bail out      */

asm {
   MOV   R0,#-7                        ; aim at LOW byte of CmdID
   LCALL ?auto0

   CLR   EA                            ; allow nice timings

   CLR   TRACE1                        ; a pair of blips to mark the start
   SETB  TRACE1
   CLR   TRACE1
   SETB  TRACE1

   SETB  DIS_EEPROM                    ; start with disabled EEPROM
   CLR   EEPROM_SK                     ; clock starts low
   CLR   EEPROM_DI                     ; set up low bit before start bit
   SETB  EEPROM_DO                     ; allow for possible data input

   CLR   DIS_EEPROM                    ; enable EEPROM
   NOP                                 ; not strictly necessary
   JB    EEPROM_DO,?EES0               ; will be low if EEPROM is jammed...

   ORL   ErrFlags,#ERR_EEPROM          ; so flag the error
   SETB  EA                            ; ensure this is on
   LJMP  ?EECan                        ;  ... and bail out now!

?EES0
   BlipHi  EEPROM_SK                   ; toggle clock on zero data bit
   NOP

   SETB  EEPROM_DI                     ; send start bit
   BlipHi  EEPROM_SK

   MOV   B,#8                          ; send command bits
   MOV   A,[R0]                        ; fetch low byte of command arg

?EESA
   RLC   A                             ; high bit first
   MOV   EEPROM_DI,C
   BlipHi  EEPROM_SK
   DJNZ  B,?EESA

   CLR   EEPROM_DI                     ; low data to get started...

}


/*--- send or receive data for read/write commands                           */
/*    when reading, the dummy zero bit does not require an extra clock       */
/*    cannot toggle heartbeat here, because EEPROM must remain enabled       */

   CLRBIT(TRACE1);
   SETBIT(TRACE1);

   switch (CmdID & 0xC0) {
   case EEP_READ :

asm {

   MOV   R0,#-4                        ; aim at HIGH byte of Data
   LCALL ?auto0

   MOV   R1,#2                         ; set up for two bytes of data

?EERd0
   MOV   B,#8                          ; set up for eight bits of data

?EERd1
   BlipHi  EEPROM_SK                   ; pump out the bit
   NOP                                 ; allow max settling time
   NOP
   NOP
   MOV   C,EEPROM_DO                   ; pick up data bit
   RLC   A                             ; tuck into accumulator
   DJNZ  B,?EERd1

   MOV   [R0],A                        ; tuck the byte away
   DEC   R0                            ; step to lower byte
   DJNZ  R1,?EERd0

}

      break;

   case EEP_WRITE :

asm {

   MOV   R0,#-4                        ; aim at HIGH byte of Data
   LCALL ?auto0

   MOV   R1,#2                         ; set up for two bytes of data

?EEWr0

   MOV   A,[R0]                        ; fetch the byte
   MOV   B,#8                          ; set up for eight bits of data

?EEWr1
   RLC   A                             ; get the next data bit
   MOV   EEPROM_DI,C                   ; send it out
   BlipHi  EEPROM_SK                   ; crank it into the chip
   DJNZ  B,?EEWr1

   DEC   R0                            ; step to lower byte

   DJNZ  R1,?EEWr0

}

      break;
   default :
      break;
   }

/* check status                                                              */
/* we rely on the fact that the input bit is pulled up by the 8031 port      */
/*  so Ready is indistinguishable from Tristated (or completely missing)     */
/*  ... so this works for commands that do not return any real status!       */

asm {

   SETB  EA                            ; restore interrupt settings

   CLR  TRACE1                         ; flag start of wait
   SETB TRACE1

   SETB  EEPROM_DO                     ; ensure we can read the input
   SETB  DIS_EEPROM                    ; indicate we want the status output

   CLR   FlagHeart                     ; tell heartbeat interrupt we are OK

   MOV   A,T0Ticks                     ; set up timeout
   ADD   A,#3                          ; about 15 ms, more or less

   CLR   DIS_EEPROM                    ; trigger the output
   NOP

?EEWt0
   JB    EEPROM_DO,?EEWt1              ; done when bit goes high
   CLR   TRACE1
   SETB  TRACE1
   CJNE  A,T0Ticks,?EEWt0              ; wait until we time out

   SETB  DIS_EEPROM
   ORL   ErrFlags,#ERR_EEPROM          ; flag the timeout
   CLR   DIS_EEPROM

?EEWt1

   CLR  TRACE1                         ; flag end of wait
   SETB TRACE1

?EECan

}

   TmrHeartBeat();

   if (ErrFlags & ERR_EEPROM) {        /* set error indication               */
      Data = 0xbad1;
   }


   SETBIT(DIS_EEPROM);                 /* ensure EEPROM is disabled          */
   SETBIT(EEPROM_SK);                  /* clock is irrelevant                */
   SETBIT(EEPROM_DI);                  /* as is the data line                */
   SETBIT(EEPROM_DO);                  /* allow other inputs here            */

   return Data;

}

/*---------------------------------------------------------------------------*/
/* Get configuration data and verify that it is OK                           */
/* If not, each Init routine will have to figure out what to do...           */

CfgGetConfig() {

BYTE Index;
WORD CheckSum;

   EEPStatus = EES_UNKNOWN;

   CheckSum = 0;
   for (Index=0; Index<EEPROMSIZE; ++Index) {
      EEImage[Index] = CfgSendEECmd(EEP_READ+Index,0);
      CheckSum += EEImage[Index];
   }

#ifdef x0
   CfgDumpState();
#endif

   if (ErrFlags & ERR_EEPROM) {                    /* timeout or jam?        */
      CmdShowMsg("\n*** EEPROM timeout or hardware failure\n");
      EEPStatus = EES_BROKEN;
      goto Clear;
   }

   if (CheckSum) {                                 /* bad checksum?          */
      CmdShowMsg("\n*** Bad EEPROM checksum\n");
      goto Clear;
   }

   if ((EEImage[EE_ID] != CfgIdent) ||             /* is it our EEPROM?      */
       (EEImage[EE_IDCPL] != -CfgIdent)) {
      CmdShowMsg("\n*** EEPROM chip from different COMM-Link 2 board!\n");
      goto Clear;
   }

   EEPStatus = EES_VALID;
   goto Done;

Clear:
   CfgClear();

Done:

   return;
}


/*---------------------------------------------------------------------------*/
/* Set configuration array to blank state                                    */

CfgClear() {

   CmdShowMsg("Initializing EEPROM data\n");

   memset(EEImage,0,sizeof(EEImage));

   EEImage[EE_ID] = CfgIdent;
   EEImage[EE_IDCPL] = -CfgIdent;

   EEPStatus = EES_EMPTY;

}

/*---------------------------------------------------------------------------*/
/* Write current configuration array to the EEPROM                           */
/* We skip this if the status is UNKNOWN, to avoid mashing a dead EEPROM     */

CfgPutConfig() {

BYTE Index;
WORD CheckSum;
WORD Changes;

   CheckSum = -EEImage[EE_CHECKSUM];   /* do not include existing sum        */
   Changes = 0;                        /* and see if anything changed        */
   for (Index=0; Index<EEPROMSIZE; ++Index) {
      CheckSum += EEImage[Index];
      Changes |= EEImage[Index] ^ EEOldImage[Index];
   }

   switch (EEPStatus) {
   case EES_UNKNOWN :                  /* never read data back               */
   case EES_BROKEN :                   /* hardware failure                   */
      goto Done;
      break;
   case EES_VALID :
      if (!Changes) {                  /* skip write if config unchanged     */
         goto Done;
      }
      break;
   case EES_EMPTY :                    /* rewrite new configuration          */
      break;
   }

   CmdShowMsg("Writing EEPROM data\n");

   EEImage[EE_CHECKSUM] = -CheckSum;   /* use complement of sum of others    */

   CfgSendEECmd(EEP_EWEN,0);           /* enable write/erase functions       */

   CLRBIT(TRACE2);                     /* flag the erase                     */
   TmrDelay100US(1);
   SETBIT(TRACE2);

   CfgSendEECmd(EEP_ERAL,0);           /* flush all entries                  */

   for (Index=0; Index<EEPROMSIZE; ++Index) {
      CLRBIT(TRACE2);                  /* flag the write action              */
      TmrDelay100US(1);
      SETBIT(TRACE2);
      CfgSendEECmd(EEP_WRITE+Index,EEImage[Index]);
   }

   CfgSendEECmd(EEP_EWDS,0);           /* disable write/erase functions      */

   if (ErrFlags & ERR_EEPROM) {                    /* hardware problem?      */
      CmdShowMsg("*** EEPROM failure, configuration data not stored\n");
   }

Done:
   move(EEOldImage,EEImage,sizeof(EEImage));       /* save for next update   */
   return;
}

