/* COMMAND.C -- command decoder                                              */


/*---------------------------------------------------------------------------*/
/* Create network address from P3 bit                                        */
/* The next-to-last character of the string gets the bits ORed in at the end */
/*  (the string should always end in a blank, but we do not check for this)  */

#ifdef NETWORK

int CmdMakeAddress() {

BYTE AddrLen;

   AddrLen = strlen(CmdNetAddress);    /* figure out target character        */

   if (AddrLen < 2) {                  /* bad address, fix it up first       */
      move(CmdNetAddress,CmdDefAddr,MAXADDRLEN+1);
      AddrLen = strlen(CmdNetAddress);
   }

   TestBits = CmdNetAddress[AddrLen - 2];          /* set up char for mods   */

asm {
   SETB  ADDRBIT                       ; allow input on bit
   MOV   C,ADDRBIT                     ; pick up
   CPL   C                             ; and flip so HIGH goes to 0
   MOV   A.0,C                         ; insert the input bit at low end
   MOV   TestBits,A                    ; set up for C code
}

   CmdNetAddress[AddrLen - 2] = TestBits;          /* install new char       */

   return;

}

#endif

/*---------------------------------------------------------------------------*/
/* Determine if we are in manual or network mode                             */
/* Sets up the control variables based on input bit                          */

#ifdef NETWORK

int CmdCheckMode() {

   SETBIT(MANUALMODE);                 /* allow inupt if shared w/ output    */
   TESTBIT(TestBits,MANUALMODE);
   if (TestBits) {
      CmdNetMode(NETMODE);             /* pin is HIGH, use default setup     */
   }
   else {
      CmdNetMode(0);                   /* pin is low, force manual mode      */
   }

   return;

}

#endif


/*---------------------------------------------------------------------------*/
/* Decode commands                                                           */

CmdDecode(pCmd)
char *pCmd;
{
char *pChar;
char *pArg;
WORD DevCode;
WORD PortValue;

   strcpy(RawBuffer,CmdBuffer);        /* preserve text for T command        */

   pChar = pArg = pCmd;                /* uppercase & squash blanks          */
   while (*pChar) {
      if (' ' == *pChar) {                         /* skip blanks here       */
         ++pChar;
      }
      else {
         *pArg = toupper(*pChar);
         ++pChar;                                  /* copy the rest          */
         ++pArg;
      }
   }
   *pArg = 0;                          /* force terminator after squash      */
   pChar = pCmd+1;                     /* set up for arg if any              */

   switch (*pCmd) {
   case 'A' :                          /* dump or set network address        */
      CmdDecSetAddr(pCmd);
      break;
   case 'D' :                          /* dump internal or stored buffers    */
      CmdDecDumpState();
      sprintf(CmdRespBuff,"T0Ticks %u\n",T0Ticks);
      Nputstr(CmdRespBuff);
      break;
   case 'E' :                          /* show error flags                   */
      CmdDecDumpErrs(pCmd);
      break;
   case 'L' :                          /* set debug logging options          */
      CmdDecLogging(pCmd);
      break;
   case 'N' :                          /* select net mode                    */
      CmdDecNetwork(pCmd);
      break;
   case 'Q' :                          /* query input values                 */
      if (!IOGetValue(&pChar,&DevCode,&PortValue)) {
         IOFormatData(DevCode,PortValue);
      }
      Nputch('\n');
      break;
   case 'R' :
      if (strcmp(pChar,"ESET")) {                  /* nonzero = no match     */
          if (isdigit(*pChar)) {
             CharDelay = atoi(pChar);
          }
          else {
             sprintf(CmdRespBuff,"Char delay is %u x 100 us\n",CharDelay);
             Nputstr(CmdRespBuff);
          }
      }
      else {
         SerWaitOutDone();
 asm {
   LJMP   0                            ; solid reset...
 }
      }
      break;
   case 'S' :                          /* set output value                   */
      while (*pChar) {
         if (IOSetValue(&pChar,&DevCode)) {        /* handles clk strings    */
            break;
         }
         if (',' == *pChar) {          /* step over separator                */
            ++pChar;
         }
      }
      break;
   case 'T' :                          /* send text string                   */
      DevCode = IOParseDev(&pChar);
      if (NODEVSPEC != DevCode) {
         pChar = strchr(RawBuffer,'=');            /* locate in saved string */
         if (pChar && ('=' == *pChar) && (1 < strlen(pChar))) {
            IOSendString(pChar+1,DevCode);
         }
         else {
            CmdShowMsg("*** T command string is missing\n");
         }
      }
      break;
   case 0 :                            /* ignore empy line                   */
      break;
   case '?' :
      Nputstr("Available commands (values in decimal unless noted)\n");
      Nputstr(" A=string   + set network address to string\n");
      Nputstr(" D          * display internal status variables (some in hex)\n");
      Nputstr(" E          * show and clear error flags\n");
      Nputstr(" L n          set reporting mode (bitmapped)\n");
      Nputstr(" N n       *+ set network mode\n");
      Nputstr(" Q            query inputs\n");
      Nputstr(" RESET        perform power-on reset initialization\n");
      Nputstr(" R n          set character delay for T command\n");
      Nputstr(" S            set output\n");
      Nputstr(" T            send ASCII text string\n");
      Nputstr("+ marks network-only commands\n");
      Nputstr("* marks debugging commands\n");
      break;
   default :
      CmdDecNoCmd(pCmd);
   }

   return;

}

