/* TIMER.C                                                                   */

/*---------------------------------------------------------------------------*/
/* Set up timers and suchlike                                                */

TmrInitialize() {

/*--- interrupt vectors                                                      */

 asm {
TMRHERE EQU *
   ORG   $000B                   /* Timer 0 interrupt     (IR pulses)        */
   LJMP  Tmr0Tick
   ORG   TMRHERE
 }

/*--- global variables                                                       */

   TmrBits = 0;                        /* flush all flags at once            */
   T0Ticks = 0;                        /* clear fractional seconds           */

   T0Reload = T0RldNorm;               /* set up normal interrupt period     */

/*--- Set up Timer 0 for long-period timings                                 */

   TCON &= 0xCF;                 /* clear run & interrupt flags              */

   TMOD &= 0xF0;                 /* set mode 1 (16-bit reload)               */
   TMOD |= 0x01;

   TH0 = TL0 = 0;                /* first delay is a long one                */

   SETBIT(ET0);                  /* enable the interrupt                     */
   SETBIT(TR0);                  /* start it running                         */

}


/*---------------------------------------------------------------------------*/
/* Timer 0 interrupt handler                                                 */
/* This is in assembler just to save some overhead                           */

Tmr0Tick() {

;
 asm {
*  SETB  TIMERFLAG
   PUSH  A
   PUSH  PSW

*--- reload the timer

   CLR   TR0
   MOV   TL0,T0Reload
   MOV   TH0,T0Reload+1
   SETB  TR0

*--- update counter and check for a full second

   INC   TickCtr                       ; increment general timer
   INC   T0Ticks                       ; tick the wrap counter

   MOV   A,T0Ticks                     ; second yet?
   CJB   A,#TICKSPERSEC,T0Frac

   SETB  FlagSecond                    ; yes, record this fact
   ADD   A,#-TICKSPERSEC               ; and knock off a full second
   MOV   T0Ticks,A

T0Frac   EQU   *

*--- restore regs and return...

T0End    EQU   *
   POP   PSW
   POP   A

*  CLR   TIMERFLAG
   RETI                          ; pre-empt normal C level return

 }

}


/*---------------------------------------------------------------------------*/
/* Delay for a fixed number of milliseconds                                  */
/* Zero NumTicks will wait next tick                                         */

TmrDelayTicks(NumTicks)
unsigned NumTicks;
{
BYTE TickSample;

   do {
      TickSample = T0Ticks;            /* catch current timer tick counter   */
      while (TickSample == T0Ticks);   /* wait for timer tick                */
   } while (NumTicks--);               /* count each tick                    */



}


/*---------------------------------------------------------------------------*/
/* Delay for a fixed number of 100 microsecond ticks                         */
/* Zero NumUS is special-cased to bail out                                   */

TmrDelay100US(Num100US)
unsigned Num100US;
{

   if (!Num100US) {                    /* bail out for zero input            */
      return;
   }

   do {
 asm {
      MOV   B,#46
Tmr?1 EQU   *
      DJNZ  B,Tmr?1
 }
   } while (--Num100US);



}
