/* Home Control System II                                                    */
/* LCD Terminal                                                              */
/* Copyright 1992 Circuit Cellar Inc                                         */
/* Firmware Furnace #25  Ed Nisley                                           */

/* Compiled with Micro-C using Compact model                                 */
/* External RAM must be in Code/Data space                                   */
/* Local variables are in external RAM                                       */

/* See READ.ME for details on changes to default Micro-C files               */

#include <8051reg.h>
#include <8051io.h>

#include "e:\mc\cust51\8051base.h"
#include "e:\mc\cust51\8051sfb.h"

/*---------------------------------------------------------------------------*/
/* Constants                                                                 */

#define VERSION "1.0a"

/*--- define NETWORK to produce RS-485 version of the code                   */
/*    define NONCOMMERCIAL to produce BBS version of distribution code       */
/*    these are normally contained in the build.ctl code and go like this:   */
/*  #define NETWORK          or NETWORKxxx                                   */
/*  #define NONCOMMERCIAL    or NONCOMMERCIALxxx                             */

#include "build.ctl"


/*---------------------------------------------------------------------------*/
/* Output bits driven from the command line or Ctrl-G beeper                 */

#define OUT_A        P1.6
#define OUT_B        P1.5
#define OUT_C        P1.4

#define OUT_BEEP     P1.4
#define BEEPTIME     100

#include "lcdlink.h"

#ifdef NETWORK
#include "..\network\console.h"
#include "..\network\command.h"
#else
#include "..\direct\console.h"
#include "..\direct\command.h"
#endif

#include "timer.h"
#include "lcd.h"
#include "ansi.h"

/*---------------------------------------------------------------------------*/
/* Global variables                                                          */

/*--- these start at 0008 as defined in 8051rlp1.asm                         */
/*    0008-9 is the External RAM stack pointer used by the compiler          */

NEAR BYTE ErrFlags;

#ifdef NETWORK
#include "..\network\console.d1"
#else
#include "..\direct\console.d1"
#endif

#include "timer.d1"
#include "lcd.d1"

/*--- bit variables must be located at IRAM addresses 0x20 through 0x3F      */
/*    note that bit variables from other files must be accounted for...      */

NEAR BYTE Padding[0x1F - CON_D1 - TMR_D1 - LCD_D1 - 0x08 - 3];
NEAR BYTE TestBits;                    /* 1F global temp for TESTBIT macro   */

#ifdef NETWORK
#include "..\network\console.b"
#include "..\network\command.b"
#else
#include "..\direct\console.b"
#include "..\direct\command.b"
#endif

#include "timer.b"
#include "lcd.b"

/*---------------------------------------------------------------------------*/
/* Variables in RAM and constants in EPROM                                   */
/* All initialized "variables" are located in EPROM...                       */

#ifdef NETWORK
#include "..\network\console.d2"
#include "..\network\command.d2"
#else
#include "..\direct\console.d2"
#include "..\direct\command.d2"
#endif

#include "lcd.d2"
#include "ansi.d2"

char CmdDefAddr[] = "TERM0 ";                      /* default net address    */

BYTE ReportMode;


/*---------------------------------------------------------------------------*/
/* Initializations                                                           */

Initialize() {

   ErrFlags = 0;                       /* clear error monitors               */

 asm {
   MOV   R0,SP                         ; clear the stack for bounds check
   INC   R0
   CLR   A
?ilp
   MOV   [R0],A
   INC   R0
   CJNE  R0,#LOW(MAXSTACK+1),?ilp      ; can wrap from FF to 100 just fine
 }

}

#ifdef NETWORK
#include "..\network\console.c"
#include "..\network\command.c"
#else
#include "..\direct\console.c"
#include "..\direct\command.c"
#endif

#include "timer.c"
#include "lcd.c"
#include "ansi.c"
#include "command.c"


/*---------------------------------------------------------------------------*/
/* The Main Event                                                            */

main() {

WORD CmdAddr;
BYTE AnyCmds;

   Initialize();

   TmrInitialize();              /* set up timer for heartbeat               */
   SerInitialize();              /* set up serial port hardware              */

   SETBIT(EA);                   /* start up interrupt handlers              */

   CmdInitialize();              /* set up command decoder                   */
   LCDInitialize();              /* set up LCD panel                         */
   ANSIInitialize();             /* set up ANSI decoder                      */

#ifdef NETWORK
   if (!CmdMode) {
      Nputstr("\nLCD-Link Networked LCD Terminal, Version ");
      Nputstr(VERSION);
      Nputstr("\n  Copyright 1992 Circuit Cellar, Inc.\n");
#ifdef NONCOMMERCIAL
      Nputstr("  NOT FOR COMMERCIAL USE\n");
      Nputstr("  Firmware Furnace #27,  Ed Nisley\n");
#endif
   }
#else
   Nputstr("\nLCDTerm LCD Terminal, Version ");
   Nputstr(VERSION);
   Nputstr("\n  Copyright 1992 Circuit Cellar, Inc.\n");
#ifdef NONCOMMERCIAL
   Nputstr("  NOT FOR COMMERCIAL USE\n");
   Nputstr("  Firmware Furnace #27,  Ed Nisley\n");
#endif
#endif


/*                 12345678901234567890                                      */
#ifdef NETWORK
   sprintf(CmdRespBuff,
                  "LCD-Link V%s\n",  VERSION);
#else
   sprintf(CmdRespBuff,
                  "LCD Terminal V%s\n",  VERSION);
#endif
   ANSISendString(CmdRespBuff);
   ANSISendString("(c)92 Circuit Cellar");
#ifdef NONCOMMERCIAL
   ANSISendString("Firmware Furnace #27");
   ANSISendString("NO COMMERCIAL USE ");
#endif

   CmdShowPrompt();              /* show prompt if in human mode...          */

   AnyCmds = 0;                  /* keep LCD message up until first cmd      */

   while (TRUE) {

      TESTBIT(TestBits,FlagSecond);    /* another second gone by?            */
      if (TestBits) {
         CPLBIT(MAINFLAG);
         CPLBIT(MAINFLAG);
         CPLBIT(MAINFLAG);
         CLRBIT(FlagSecond);
      }

      if (NULL != (CmdAddr = CmdGetLine())) {      /* any commands?          */
         if (!AnyCmds) {
            ANSISendString("\x1b[2J");             /* yes, clear on first    */
            AnyCmds = 1;
         }
         CmdDecode(CmdAddr);                       /* and decode them        */
         CmdShowPrompt();                          /* prompt for humans      */
      }

   }

   Nputstr("--- Fell out the bottom!\n");          /* impossible...          */

}

