/* IRINTF.H -- IR transmitter/receiver interface                             */

/*---------------------------------------------------------------------------*/
/* Galactic IR constants                                                     */

#define NUMDATABITS 9                  /* data bits encoded in each message  */
#define MAXMSGNUM   511                /* and the corresponding value        */

#define BADGE_DELAY  40                /* timer ticks for badge echo         */

/*--- Timer 0 reload values that define bit times for input and output       */
/*    bit time = 16/12.4 KHz = 1.29 ms                                       */
/*    MC145030 uses Manchester encoding                                      */
/*     transmitter ticks twice per bit = 645 us                              */
/*     receiver ticks twice per bit = 645 us                                 */
/*       (but rec must be synchronized to incoming bit stream!!!             */
/*    timer ticks are figured at (time in us)/(1.085 us/tick)                */

#define T0RldNorm        -4741                     /* 5.16 ms - 15 cycles    */
#define T0RldTrans       -580                      /* 645 us - 15 cycles     */
#define T0RldRec         -580                      /* 645 us - 15 cycles     */

#define T0RldRecSync     -287                      /* for first sync sample  */
#define T0RldRecAdj      -287                      /* reload at Int0 events  */
#define T0SyncAdj        30                        /* fudge after 1st sync   */

#define TICKSPERSEC     194                        /* 1/(ticks per second)   */
#define IRPERTICK       8                          /* IR int speedup factor  */

/*--- I/O bit locations                                                      */

#define IRECEIVE     INT0                          /* hardcoded in IRINTF    */
#define ITRANS       INT1                          /* + active               */

#define IRECIRQ      IE0                           /* hardcoded in IRINTF    */

#define OUT_A        P1.6                          /* output bits            */
#define OUT_B        P1.5
#define OUT_C        P1.4

#define IRFREQMON    T1                            /* Transmitter freq input */

