/* POLLING.C -- IR badge tracking                                            */

#define POLL_TRACE

/*---------------------------------------------------------------------------*/
/* Set up timers and suchlike                                                */

PolInitialize() {

   PolNumIDs = 0;                      /* disable polling                    */
   PolNextID = 0;

   PollDly = 0;                        /* polling is disabled                */
   PollInterval = POLL_INTERVAL;       /* but will run at this rate          */

}


/*---------------------------------------------------------------------------*/
/* Decide whether it's time to send another poll                             */
/* If so, dump the next badge number into the transmitter ring               */

PolCheck() {

   if ((!PollDly) && PolNumIDs) {      /* ready and enabled?                 */

      if (!IRSendMsg(PolNextID)) {     /* fire it off, check for success     */
         if (REPORT_POLL & ReportMode) {
            sprintf(CmdRespBuff,"Pol %u [%04x hex]\n",PolNextID,PolNextID);
            Nputstr(CmdRespBuff);
         }

         ++PolNextID;                  /* tick and wrap badge ID             */
         if (PolNumIDs <= PolNextID) {
            PolNextID = 0;             /* restart from badge zero            */
         }

         PollDly = PollInterval;       /* reset delay to next poll           */
      }

   }

}


/*---------------------------------------------------------------------------*/
/* Dump current polling status                                               */

PolDumpState() {

   Nputstr("Badge polling status:\n");

   sprintf(CmdRespBuff,
           " PolNumIDs %u, PolNextID %u, PollDly %u, PollInterval %u\n",
            PolNumIDs,PolNextID,PollDly,PollInterval);
   Nputstr(CmdRespBuff);

}


