/* IR signal sample & display routine                                        */
/* Copyright 1992 Circuit Cellar Inc                                         */
/* Firmware Furnace #29  Ed Nisley                                           */

/* Compiled with Micro-C using Compact model                                 */
/* External RAM must be in Code/Data space                                   */
/* Local variables are in external RAM                                       */

/* Runs on IR-Link board as described in Circuit Cellar INK 26               */
/* Requires additional IR photodiode and LM311 comparator                    */
/*  connected to T1 input pin                                                */

#include <8051reg.h>
#include <8051io.h>

#define BYTE unsigned char
#define WORD unsigned int

#define LOW(x) x
#define HIGH(x) =x

/*---------------------------------------------------------------------------*/
/* Constants                                                                 */

#define IR_INPUT P3.5                              /* aka T1                 */

#define BYTES_PER_LINE  7

/*---------------------------------------------------------------------------*/
/* Buffer for IR samples                                                     */
/* We actually use only 255 bytes starting on the first 0x100 boundary       */
/* Remember that a 1 bit is OFF and a 0 bit is ON                            */

BYTE IRBuffer[1000];

/*---------------------------------------------------------------------------*/
/* Tickmarks for timing reference                                            */

char TickMarks[101];
char TickBuffer[101];


/*---------------------------------------------------------------------------*/
/* The main event                                                            */

main() {
BYTE *pBase;
BYTE *pEntry;
int  Counter;
int  ByteCtr;
int  BitCtr;
WORD BitHist;
WORD OldHist;
BYTE EdgeCtr;
BYTE EdgeMark;


   serinit(9600);                /* set up serial port interface             */

   putstr("\nIR Signal Sampler Version 1.0a\n");
   putstr("Copyright 1992 Circuit Cellar, Inc.\n");
   putstr("  NOT FOR COMMERCIAL USE\n");
   putstr("Firmware Furnace #29, Ed Nisley\n\n");

   putstr("Connect IR photodiode/LM311 comparator to T1,\n");
   putstr("  adjust trimpot so LED is OFF\n");

   memset(TickMarks,'\xFA',sizeof(TickMarks)-1);
   TickMarks[0] = '\xC3';
   for (Counter=5; Counter < sizeof(TickMarks)-6; Counter += 10) {
      TickMarks[Counter] = '\xC1';
      TickMarks[Counter+5] = '\xC5';
   }

   while (1) {


      do {
         putstr("--- Press Enter to start sampling...\n");
      } while ('\n' != getch());

      putstr("Fire when ready...\n\n\n");

asm {

   MOV   A,#HIGH(IRBuffer)             ; point to buffer
   INC   A                             ; align to next 256 byte boundary
   MOV   P2,A
   MOV   R0,#$FF                       ; start from the top

?IRWait
   JB    IR_INPUT,?IRWait              ; wait for input bit

?IRTop
   MOV   C,IR_INPUT                    ; 1 fetch input
   RLC   A                             ; 1 add to byte
   MOV   C,IR_INPUT                    ; repeat for full byte
   RLC   A                             ; at two cycles per sample
   MOV   C,IR_INPUT
   RLC   A
   MOV   C,IR_INPUT
   RLC   A
   MOV   C,IR_INPUT
   RLC   A
   MOV   C,IR_INPUT
   RLC   A
   MOV   C,IR_INPUT
   RLC   A
   MOV   C,IR_INPUT
   RLC   A

   MOVX  [R0],A                        ; 2 save in external RAM
   DJNZ  R0,?IRTop                     ; 2 step to next sample

}

      putstr("One sample = 2 x 1.085 us = 2.17 us @ 11.0592 MHz\n");
      putstr("Cycles\n\n");

      pBase = (IRBuffer + 0x0100) | 0x00FF;        /* point to first sample  */
      OldHist = 0;                                 /* old bit is ON (0)      */

      Counter = 0;                                 /* samples to display     */
      EdgeCtr = 0;                                 /* up-going edge number   */
      EdgeMark = 0;                                /* no transition yet      */
      do {

/*--- top line of waveform                                                   */

         pEntry = pBase;
         BitHist = OldHist;

         putstr("     ");
         for (ByteCtr=0;
              (ByteCtr < BYTES_PER_LINE) && ((Counter + ByteCtr) < 255);
              ++ByteCtr) {
            BitHist |= (WORD)*(pEntry--);

            for (BitCtr=0; BitCtr < 8; ++BitCtr) {
               BitHist = BitHist << 1;
               switch (BitHist & 0x0300) {
               case 0x0000 :                       /* both samples ON        */
                  putch(' ');
                  break;
               case 0x0100 :                       /* just went OFF          */
                  putch('\xDA');
                  break;
               case 0x0200 :                       /* just went ON           */
                  putch('\xBF');
                  break;
               case 0x0300 :                       /* both samples OFF       */
                  putch('\xC4');
                  break;
               }
            }

            putstr("  ");                          /* skipped slots          */

         }

         putch('\n');

/*--- middle line                                                            */

         pEntry = pBase;
         BitHist = OldHist;

         putstr("     ");
         for (ByteCtr=0;
              (ByteCtr < BYTES_PER_LINE) && ((Counter + ByteCtr) < 255);
              ++ByteCtr) {
            BitHist |= (WORD)*(pEntry--);

            for (BitCtr=0; BitCtr < 8; ++BitCtr) {
               BitHist = BitHist << 1;
               switch (BitHist & 0x0300) {
               case 0x0000 :                       /* both samples ON        */
                  putch(' ');
                  break;
               case 0x0100 :                       /* just went OFF          */
                  EdgeMark = 1;
                  printf("%02d",EdgeCtr);          /* use exactly two chars  */
                  ++EdgeCtr;
                  if (EdgeCtr > 99) {
                     EdgeCtr = 0;
                  }
                  break;
               case 0x0200 :                       /* just went ON           */
                  putch('\xB3');
                  break;
               case 0x0300 :                       /* both samples OFF       */
                  if (EdgeMark) {
                     EdgeMark = 0;                 /* skip blank for number  */
                  }
                  else {
                     putch(' ');
                  }
                  break;
               }
            }

            putstr("  ");                          /* skipped slots          */

         }

         putch('\n');

/*--- bottom line                                                            */

         pEntry = pBase;
         BitHist = OldHist;

         putstr("     ");
         for (ByteCtr=0;
              (ByteCtr < BYTES_PER_LINE) && ((Counter + ByteCtr) < 255);
              ++ByteCtr) {
            BitHist |= (WORD)*(pEntry--);

            for (BitCtr=0; BitCtr < 8; ++BitCtr) {
               BitHist = BitHist << 1;
               switch (BitHist & 0x0300) {
               case 0x0000 :                       /* both samples ON        */
                  putch('\xC4');
                  break;
               case 0x0100 :                       /* just went OFF          */
                  putch('\xD9');
                  break;
               case 0x0200 :                       /* just went ON           */
                  putch('\xC0');
                  break;
               case 0x0300 :                       /* both samples OFF       */
                  putch(' ');
                  break;
               }
            }

            putstr("  ");                          /* skipped slots          */

         }

         putch('\n');

/*--- index tickmarks                                                        */

         printf("%4d ",Counter*20);                /* 2 x 10 x samples       */

         move(TickBuffer,TickMarks,BYTES_PER_LINE*10);
         TickBuffer[BYTES_PER_LINE*10] = 0;
         putstr(TickBuffer);

         putch('\n');
         putch('\n');


/*--- clean up for next group                                                */

         pBase -= BYTES_PER_LINE;
         Counter += BYTES_PER_LINE;
         OldHist = BitHist;

      } while (Counter < 255);

   }

   putstr("--- Fell out the bottom!\n");           /* impossible...          */

}

