asm {

*----------
* Macros
* Uses XASM MACRO preprocessor
*  tweaked to be included as a C header file!
* Remember that MOV A,#word gets low byte, MOV A,#=word gets high byte

*---
* C level macros do not affect what goes on here, so a few bits are in order

F0FLAG        SET       PSW.5

*--- send an internal bit out a port bit
*    ShowB portbit,intbit
*
*ShowB         MACRO
*
*              SETB  \1
*              MOV   C,\2
*              MOV   \1,C
*              CLR   \1
*
*              ENDMAC

*--- send ACC out a port bit
*
*DumpACC       MACRO
*
*              CLR       \1
*              RLC       A
*              MOV       \1,C
*              SETB      \1
*              CLR       \1
*              RLC       A
*              MOV       \1,C
*              SETB      \1
*              CLR       \1
*              RLC       A
*              MOV       \1,C
*              SETB      \1
*              CLR       \1
*              RLC       A
*              MOV       \1,C
*              SETB      \1
*              CLR       \1
*              RLC       A
*              MOV       \1,C
*              SETB      \1
*              CLR       \1
*              RLC       A
*              MOV       \1,C
*              SETB      \1
*              CLR       \1
*              RLC       A
*              MOV       \1,C
*              SETB      \1
*              CLR       \1
*              RLC       A
*              MOV       \1,C
*              SETB      \1
*              CLR       \1
*              RLC       A
*              SETB      \1
*
*              ENDMAC

*--- pulse a bit high for three cycles
*    BlipHi bit

BlipHi        MACRO

              SETB      \1
              NOP
              NOP
              CLR       \1

              ENDMAC

*--- pulse a bit low for three cycles
*    BlipLo bit
*
*BlipLo        MACRO
*
*              CLR       \1
*              NOP
*              NOP
*              SETB      \1
*
*              ENDMAC
*
*--- pulse a bit for three cycles
*    Pulse bit

Pulse         MACRO

              CPL       \1
              NOP
              NOP
              CPL       \1

              ENDMAC

*--- toggle an output bit 'n' times
*    Blip bit,reps

Blip          MACRO

              CPL       \1
              PUSH      B
              MOV       B,#3
ndly\$        DJNZ      B,ndly\$

              MOV       B,\2
next\$        EQU       *
              CPL       \1
              CPL       \1
              DJNZ      B,next\$

              MOV       B,#3
fdly\$        DJNZ      B,fdly\$

              POP       B
              CPL       \1

              ENDMAC

*--- the missing XRL for bits...
*     XRLB  bit1,bit2
*     leaves result in C
*     stamps on F0FLAG, which may induce peculiar bugs

XRLB          MACRO

              MOV       C,\1
              ANL       C,/\2
              MOV       F0FLAG,C

              MOV       C,\2
              ANL       C,/\1
              ORL       C,F0FLAG

              ENDMAC


*--- Rotate a 4-byte thing left by one bit
*    Assumes high-order byte is in low-order address as usual
*    leaves bit just shifted out in the C bit if that is of interest

RL32          MACRO

              MOV       A,\1+3              ; start with low byte
              RLC       A
              MOV       \1+3,A
              MOV       A,\1+2
              RLC       A
              MOV       \1+2,A
              MOV       A,\1+1
              RLC       A
              MOV       \1+1,A
              MOV       A,\1                ; finish up with high byte
              RLC       A
              MOV       \1,A
              MOV       A,\1+3              ; put high bit in low byte
              MOV       A.0,C
              MOV       \1+3,A

              ENDMAC


*--- Select a register bank

SetRBank      MACRO

             IFEQU      \1,0
              CLR       RS1
              CLR       RS0
             ENDIF

             IFEQU      \1,1
              CLR       RS1
              SETB      RS0
             ENDIF

             IFEQU      \1,2
              SETB      RS1
              CLR       RS0
             ENDIF

             IFEQU      \1,3
              SETB      RS1
              SETB      RS0
             ENDIF


              ENDMAC

*--- get a byte from Code Space

GetCData      MACRO

             IFNE       \2,' '
              PUSH      DPH
              PUSH      DPL
             ENDIF

              MOV       DPTR,#\1
              CLR       A
              MOVC      A,[A+DPTR]

             IFNE       \2,' '
              POP       DPL
              POP       DPH
             ENDIF

              ENDMAC

*--- get a word from Code Space into DPTR
*    presumes the word is stored in "DW" format:  high first
*
*GetCWord      MACRO
*
*              PUSH      A                   ; save bystander
*
*              MOV       DPTR,#\1
*              CLR       A
*              MOVC      A,[A+DPTR]
*              PUSH      A
*
*              INC       DPTR
*              CLR       A
*              MOVC      A,[A+DPTR]
*
*              MOV       DPL,A
*              POP       DPH
*
*              POP       A
*
*             ENDMAC

*--- get a byte from Code Space table into ACC
*
*GetCTable     MACRO
*
*             IFNE      \2,' '
*              PUSH      DPH
*              PUSH      DPL
*             ENDIF
*
*              MOV       DPTR,#\1
*              MOVC      A,[A+DPTR]
*
*             IFNE      \2,' '
*              POP       DPL
*              POP       DPH
*             ENDIF
*
*              ENDMAC

*--- get a byte from External Data Space
*
*GetXData      MACRO
*
*             IFNE      \2,' '
*              PUSH      DPH
*              PUSH      DPL
*             ENDIF
*
*              MOV       DPTR,#\1
*              MOVX      A,[DPTR]
*
*             IFNE      \2,' '
*              POP       DPL
*              POP       DPH
*             ENDIF
*
*              ENDMAC

*--- get a word from Data Space into DPTR
*    presumes the word is stored in "DW" format:  high first

GetXWord      MACRO

              PUSH      A                   ; save bystander

              MOV       DPTR,#caddr
              MOVX      A,[DPTR]
              PUSH      A

              INC       DPTR
              MOVX      A,[DPTR]

              MOV       DPL,A
              POP       DPH

              POP       A

              ENDMAC

*--- put a byte into External Data Space
*
*PutXData      MACRO
*
*             IFNE      \2,' '
*              PUSH      DPH
*              PUSH      DPL
*             ENDIF
*
*              MOV       DPTR,#\1
*              MOVX      [DPTR],A
*
*             IFNE      \2,' '
*              POP       DPL
*              POP       DPH
*             ENDIF
*
*              ENDMAC

*--- put DPTR into External Data Space
*
*PutXWord      MACRO
*
*              PUSH      A                   ; save bystander
*
*              PUSH      DPL                 ; save the word
*              PUSH      DPH
*
*              MOV       DPTR,#\1         ; aim at the target
*              POP       A                   ; recover high byte
*              MOVX      [DPTR],A
*              INC       DPTR
*              POP       A                   ; recover low byte
*              MOVX      [DPTR],A
*              INC       DPTR                ; step beyond last byte
*
*              POP       A                   ; restore bystander
*
*              ENDMAC

*--- put a byte from B into a table at xaddr+ACC
*
*PutXTable     MACRO
*
*             IFNE      \2,' '
*              PUSH      DPH
*              PUSH      DPL
*             ENDIF
*
*              ADD       A,#\1
*              MOV       DPL,A
*              MOV       A,#=\1
*              ADDC      A,#0
*              MOV       DPH,A
*              MOV       A,B
*              MOVX      [DPTR],A
*
*             IFNE      \2,' '
*              POP       DPL
*              POP       DPH
*             ENDIF
*
*              ENDMAC

*--- get a byte from table at xaddr+ACC
*
*GetXTable     MACRO
*
*             IFNE      \2,' '
*              PUSH      DPH
*              PUSH      DPL
*             ENDIF
*
*              ADD       A,#\1
*              MOV       DPL,A
*              MOV       A,#=\1
*              ADDC      A,#0
*              MOV       DPH,A
*              MOVX      A,[DPTR]
*
*             IFNE      \2,' '
*              POP       DPL
*              POP       DPH
*             ENDIF
*
*              ENDMAC

*--- decrement that knows about DPTR...

DecDPTR       MACRO

              XCH       A,DPL
              JNZ       dlow\$
              DEC       DPH
dlow\$        DEC       A
              XCH       A,DPL

              ENDMAC

*--- push & pop DPTR

PushDPTR      MACRO
              PUSH      DPL
              PUSH      DPH
              ENDMAC

PopDPTR       MACRO
              POP       DPH
              POP       DPL
              ENDMAC

*--- add to DPTR
*    presumes value is exactly a byte long

AddDPTR       MACRO

             IFNE      \1,A
              PUSH      A
              MOV       A,\1
             ENDIF

              ADD       A,DPL
              MOV       DPL,A
              CLR       A
              ADDC      A,DPH
              MOV       DPH,A

             IFNE      \1,A
              POP       A
             ENDIF

              ENDMAC


*--- decrement Ra:Rb

DecRR         MACRO

              XCH       A,\2
              JNZ       dlow\$
              DEC       \1
dlow\$        DEC       A
              XCH       A,\2

              ENDMAC


*--- increment Ra:Rb

IncRR         MACRO

              XCH       A,\2
              INC       A
              JNZ       done\$
              INC       \1
done\$        XCH       A,\2

              ENDMAC

*--- add Rc:Rd to Ra:Rb, result to Ra:Rb

AddRR         MACRO

              XCH       A,\2
              ADD       A,\4
              XCH       A,\2

              XCH       A,\1
              ADDC      A,\3
              XCH       A,\1

              ENDMAC

*--- add ACC to a register
*    (we do not check for "a" being other than the accumulator...

AddR          MACRO

              XCH       \2,\1
              ADD       \2,\1
              XCH       \2,\1

              ENDMAC

*--- subtract Rc:Rd from Ra:Rb, result to Ra:Rb

SubRR         MACRO

              XCH       A,\2
              CLR       C
              SUBB      A,\4
              XCH       A,\2

              XCH       A,\1
              SUBB      A,\3
              XCH       A,\1

              ENDMAC

*--- long jump if Ra:Rb is zero

LJZRR         MACRO

              MOV       A,\1
              ORL       A,\2
              JNZ       nT\$
              LJMP      \3

nT\$          EQU       *

              ENDMAC

*--- jump if Ra:Rb is zero

JZRR          MACRO

              MOV       A,\1
              ORL       A,\2
              JZ        \3

              ENDMAC

*--- long jump if Ra:Rb is not zero

LJNZRR        MACRO

              MOV       A,\1
              ORL       A,\2
              JZ        zT\$
              LJMP      \3

zT\$          EQU       *

              ENDMAC

*--- jump if Ra:Rb is not zero

JNZRR         MACRO

              MOV       A,\1
              ORL       A,\2
              JNZ       \3

              ENDMAC

*--- (long) positive conditional test

CJE           MACRO

              CJNE      \1,\2,nT\$
              LJMP      \3
nT\$          EQU       *
              ENDMAC

*--- long CJNE

LCJNE         MACRO

              CJNE      \1,\2,nT\$
              SJMP      xT\$
nT\$          LJMP      \3
xT\$          EQU       *

              ENDMAC

*--- long DJNZ

LDJNZ         MACRO

              DJNZ      \1,nT\$
              SJMP      xT\$
nT\$          LJMP      \2
xT\$          EQU       *

              ENDMAC

*--- long JB

LJB           MACRO

              JNB       \1,nT\$
              LJMP      \2
nT\$          EQU       *
              ENDMAC

*--- long JBC

LJBC          MACRO

              JNB       \1,nT\$
              CLR       \1
              LJMP      \2
nT\$          EQU       *
              ENDMAC

*--- long JNB

LJNB          MACRO

              JB        \1,nT\$
              LJMP      \2
nT\$          EQU       *
              ENDMAC

*--- long JC

LJC           MACRO

              JNC       nT\$                  ; inverted test
              LJMP      \1
nT\$          EQU       *
              ENDMAC

*--- long JNC

LJNC          MACRO

              JC        nT\$                  ; inverted test
              LJMP      \1
nT\$          EQU       *
              ENDMAC

*--- long JZ

LJZ           MACRO

              JNZ       nT\$                  ; inverted test
              LJMP      \1
nT\$          EQU       *
              ENDMAC

*--- long JNZ

LJNZ          MACRO

              JZ        nT\$                  ; inverted test
              LJMP      \1
nT\$          EQU       *
              ENDMAC

*--- indirect CALL through DPTR with A as offset within table
*    presumes DPTR is already set up & A is doubled...

CallDPTR      MACRO

              MOV       A,#LOW  RetAddr\$   ; set up for RET
              PUSH      A
              MOV       A,#HIGH RetAddr\$
              PUSH      A

              MOV       A,\1
              JMP       [A+DPTR]

RetAddr\$     EQU       *                   ; return point
              ENDMAC

*--- comparison operators
*    these use CJNE with dummy targets to sort things out
*    C is set if first < second in unsigned comparison
*    "above" and "below" are first with respect to second...

CJB           MACRO
              CJNE      \1,\2,nT\$
nT\$          JC        \3
              ENDMAC

LCJB          MACRO
              CJNE      \1,\2,nT1\$
nT1\$         JNC       nT2\$                  ; inverted test
              LJMP      \3
nT2\$         EQU       *
              ENDMAC

CJAE          MACRO
              CJNE      \1,\2,nT\$
              SJMP      \3
nT\$          JNC       \3
              ENDMAC

CJA           MACRO
              CJNE      \1,\2,nT\$
              SJMP      end\$
nT\$          JNC       \3
end\$         EQU       *
              ENDMAC

CJBE          MACRO
              CJNE      \1,\2,nT\$
              SJMP      \3
nT\$          JC        \3
              ENDMAC

*--- simply delay loop

Delay         MACRO
              PUSH      A
              MOV       A,\1
dly\$         DJNZ      A,dly\$
              POP       A
              ENDMAC

*--- wait for a bit to go on, then clear it

WaitON        MACRO
wait\$        JNB       \1,wait\$
              CLR       \1
              ENDMAC

*--- load Timer 0 with a 16-bit value

LoadT0        MACRO
              CLR       TCON.4              * TR0
              MOV       TH0,#=-(\1)
              MOV       TL0,#-(\1)
              CLR       TCON.5              * TF0
              ENDMAC

}
