/* POST.C                                                                    */
/* Power-on diagnostics for 8031 systems                                     */
/* Return 0 if OK, nonzero if failure                                        */
/* Tickles HEARTBEAT output to prevent premature resets...                   */
/*  Modified to avoid CPL instuction to prevent latching inputs...           */


/*-------------                                                              */
/* Calculate EPROM checksum                                                  */
/* Assumes HEXFMT inserted the 16-bit checksum in a spot that had FF FF      */
/* Size may be zero, in which case we check all 64K bytes                    */

WORD TstEPROM(Size,SumAddr)
WORD Size;
WORD SumAddr;
{

   Size;
   SumAddr;

asm {

              SETB      HEARTBEAT

*--- fetch parameters

              MOV       R0,#-7              ; aim at first parameter
              LCALL     ?auto0
              MOV       2,[R0]              ; fetch low Size
              INC       R0
              MOV       3,[R0]              ; fetch high Size
              INC       R0
              MOV       DPL,[R0]            ; fetch low SumAddr
              INC       R0
              MOV       DPH,[R0]            ; fetch high SumAddr

              CLR       A                   ; fetch checksum (stored HIGH/LOW)
              MOVC      A,[A+DPTR]
              MOV       R5,A                ; HIGH byte
              MOV       A,#1
              MOVC      A,[A+DPTR]          ; LOW byte
              MOV       R4,A

*--- adjust checksum so result will equal the EPROM total value
*     observed sum = checksum + cshigh + cslow - FF - FF

              ADD       A,R5                ; high and low added together
              MOV       B,#0
              MOV       B.0,C
              ADD       A,R4                ; combine with checksum
              MOV       R4,A
              MOV       A,B
              ADDC      A,R5
              MOV       R5,A

              MOV       A,R4                ; subtract 2*FF from chksum
              ADD       A,#$02              ;  (add -2*FF = FE02)
              MOV       R4,A
              MOV       A,#$FE
              ADDC      A,R5
              MOV       R5,A

*--- add up the EPROM contents

              CLR       A                   ; set up pointer
              MOV       DPL,A
              MOV       DPH,A
              MOV       R6,A                ; and EPROM total
              MOV       R7,A

?ROMlp        EQU       *

              CLR       A                   ; fetch EPROM byte
              MOVC      A,[A+DPTR]
              ADD       A,R6                ; accumulate total
              MOV       R6,A
              CLR       A
              ADDC      A,R7
              MOV       R7,A

              CLR       HEARTBEAT

              INC       DPTR                ; step to next byte

              MOV       A,R2                ; pointer = count?
              XRL       A,DPL               ;  (we started from 0000)
              JNZ       ?ROMlp              ; no match on LSB -> continue

              SETB      HEARTBEAT           ; blip output bit each 256 LSBs
              MOV       A,R3
              XRL       A,DPH
              JNZ       ?ROMlp              ; no match on MSB -> continue

*--- compare it with the expected value...

              MOV       A,R6
              XRL       A,R4
              MOV       B,A
              MOV       A,R7
              XRL       A,R5

              CLR       HEARTBEAT

              XCH       A,B                 ; set up return value

              RET

}

}


/*-------------
/* Test all of External RAM space                                            */
/* This is a simple nondestructive test                                      */
/* Size may be zero, in which case we test a full 64K bytes                  */
                                                                             */
WORD TstRAM(Base,Size)
WORD Base;
WORD Size;
{

   Base;
   Size;

asm {

              SETB      HEARTBEAT

*--- fetch parameters

              MOV       R0,#-7              ; aim at first parameter
              LCALL     ?auto0
              MOV       DPL,[R0]            ; fetch low Base
              INC       R0
              MOV       DPH,[R0]            ; fetch high Base
              INC       R0
              MOV       2,[R0]              ; fetch low Size
              INC       R0
              MOV       3,[R0]              ; fetch high Size

*--- verify that each location will do something useful

?RAMlp        EQU       *

              CLR       HEARTBEAT

              MOVX      A,[DPTR]            ; fetch existing byte
              MOV       R4,A

              PUSH      IE                  ; save interrupt status
              CLR       IE.7                ; and disable everything

              CPL       A                   ; see if complement sticks
              MOVX      [DPTR],A
              NOP
              MOVX      A,[DPTR]
              CPL       A
              CJNE      A,4,?RAMng

              CLR       A                   ; try a zero byte
              MOVX      [DPTR],A
              NOP
              MOVX      A,[DPTR]
              JNZ       ?RAMng

              MOV       A,#$55              ; try patterns
              MOVX      [DPTR],A
              NOP
              MOVX      A,[DPTR]
              CJNE      A,#$55,?RAMng

              MOV       A,#$AA              ;  ... more
              MOVX      [DPTR],A
              NOP
              MOVX      A,[DPTR]
              CJNE      A,#$AA,?RAMng

              MOV       A,R4                ; restore original byte
              MOVX      [DPTR],A
              NOP
              MOVX      A,[DPTR]
              CJNE      A,4,?RAMng

              SETB      HEARTBEAT

              POP       IE                  ; restore interrupts

              INC       DPTR                ; step pointer

              MOV       A,R2                ; and decrement counter
              CLR       C
              SUBB      A,#1
              MOV       R2,A
              MOV       A,R3
              SUBB      A,#0
              MOV       R3,A

              MOV       A,R2                ; counter zero?
              JNZ       ?RAMlp              ; LSB not zero -> not done yet
              CPL       HEARTBEAT           ; flip heartbeat every 256 LSBs
              MOV       A,R3
              JNZ       ?RAMlp              ; MSB not zero -> not done yet

              SJMP      ?RAMdn              ; LSB & MSB zero -> done now

*--- bad compare, so attempt to restore original byte and bail out

?RAMng        EQU       *
              POP       IE                  ; restore interrupts

              MOV       A,R4
              MOVX      [DPTR],A            ; may not really work...

*--- finished, return current count
*     which will be zero if every byte passed muster

?RAMdn        EQU       *
              MOV       A,R2                ; return count, zero if all OK
              MOV       B,R3

              RET

}

}

