/* My standard INCLUDEs for Micro-C code                                     */

/*---------------------------------------------------------------------------*/
/* Memory space identifiers                                                  */
/* Micro-C slightly abuses the "register" keyword...                         */

#define NEAR register
#define FAR

/*---------------------------------------------------------------------------*/
/* Handy stuff                                                               */

#define BYTE unsigned char
#define WORD unsigned int

#define TRUE 1
#define FALSE !TRUE


/*---------------------------------------------------------------------------*/
/* Bit test function                                                         */
/* MC can't do an 'if' directly on an 8051 bit variable                      */
/*  and the optimizer does not with with IRAM bit values                     */

/* This requires a global variable, which can be trouble in an IRQ handler   */

#define TESTBIT(byte,bit) \
 asm {             \
   CLR A           \
   MOV B,A         \
   MOV C,bit       \
   MOV A.0,C       \
   MOV byte,A      \
 }

/* This leaves the result in B:A for use with a null function like nargs()   */
/*  which the optimizer will edit right out of the ensuing assembly code!    */

#define IFBIT(bit) \
 asm {             \
   CLR A           \
   MOV B,A         \
   MOV C,bit       \
   MOV A.0,C       \
 }                 \
   if (nargs())

#define IFNBIT(bit) \
 asm {              \
   CLR A            \
   MOV B,A          \
   MOV C,bit        \
   CPL C            \
   MOV A.0,C        \
 }                  \
   if (nargs())

#define GETBIT(bit) \
 asm {             \
   CLR A           \
   MOV B,A         \
   MOV C,bit       \
   MOV A.0,C       \
 }

#define BITVAL() nargs()


/*---------------------------------------------------------------------------*/
/* Bit and register values                                                   */
/* These are useful only for assembler code...                               */

/*--- useful character definitions                                           */
/*    these are in decimal so as to be useful in either C or assembler       */

#ifndef CANCEL

#define CANCEL    3
#define EOT       4
#define BELL      7
#define BS        8
#define TAB       9
#define LF        10
#define NL        10
#define FORMFEED  12
#define CR        13
#define ESC       27
#define BLANK     ' '
#define DEL       127

#define XON       17
#define XOFF      19

#endif

#ifndef EOF
#define EOF -1
#endif

/*---------------------------------------------------------------------------*/
/* Assembler macros                                                          */

#define LOW(x) x
#define HIGH(x) =x
