/* COMMAND.C -- command decoder                                              */

/* 07 Aug 92 EEN limit CSErrCount to 29999                                   */

/*---------------------------------------------------------------------------*/
/* Compare two strings up to a fixed length                                  */
/* Returns <0 if string 1 < string 2                                         */
/*          0             =                                                  */
/*         >0             >                                                  */

int strncmp(pStr1,pStr2,MaxLen)
char *pStr1;
char *pStr2;
int  MaxLen;
{
int RetVal;

   RetVal = 0;

   while ((!RetVal) && (MaxLen-- > 0)) {
      RetVal = (int)*(pStr1++) - (int)*(pStr2++);
   }

   return RetVal;

}



/*---------------------------------------------------------------------------*/
/* Set up address check bytes from known-good string                         */
/* We include the terminating byte in the checksum                           */

CmdSetAddrChk(pTarget)
char *pTarget;
{

BYTE Index;

   for (Index=0; Index<=MAXADDRLEN; ++Index) {
      *(pTarget+Index) = ~CmdNetAddress[Index];
   }
   CmdAddrOK = 1;

}




/*---------------------------------------------------------------------------*/
/* Initialize command decoder                                                */

CmdInitialize() {

BYTE Index;

   CmdBits = 0;                        /* clear all the flags                */
   CSErrCount = 0;

   CmdLineLen = 0;
   memset(CmdBuffer,0,sizeof(CmdBuffer));

   CmdRespLen = 0;
   memset(CmdRespBuff,0,sizeof(CmdBuffer));

   CmdCSLength = 0;
   memset(CmdCSBuffer,0,sizeof(CmdCSBuffer));

/*--- verify the NVRAM network address                                       */

#ifdef COMMLINK2

   Index;                              /* not used otherwise...              */

   switch (EEPStatus) {
   case EES_VALID :                    /* use address string from EEPROM     */
      memset(CmdNetAddress,0,sizeof(CmdNetAddress));
      move(CmdNetAddress,EEImage,MAXADDRLEN);
      break;
   case EES_EMPTY:                     /* EEPROM needs initialization        */
   case EES_UNKNOWN :                  /* EEPROM not read yet...             */
   case EES_BROKEN :                   /* hardware problem                   */
      memset(CmdNetAddress,0,sizeof(CmdNetAddress));
      strcpy(CmdNetAddress,CmdDefAddr);
      move(EEImage+EE_NETADDR,CmdNetAddress,MAXADDRLEN);
      break;
   }

#else

   CmdAddrOK = 1;                                  /* assume it's OK         */
   for (Index=0; Index<MAXADDRLEN; ++Index) {
      if (CmdNetAddress[Index] != ~CmdNetAddrChk[Index]) {
         CmdAddrOK = 0;
         break;
      }
   }

   if (!CmdAddrOK) {             /* no good, so set up default address       */
      memset(CmdNetAddress,0,sizeof(CmdNetAddress));
      strcpy(CmdNetAddress,CmdDefAddr);
   }

   CmdMakeAddress();             /* check port for new address selection     */

#endif

   CmdCheckMode();               /* figure out manual or network mode        */
   CmdSetAddrChk(CmdNetAddrChk); /* recompute check value for next reset     */
}


/*---------------------------------------------------------------------------*/
/* Switch network mode ON or OFF depending on argument                       */

CmdNetMode(State)
int State;
{

   SerWaitOutDone();             /* wait for serial port to finish...        */

   CmdMode = State;

   CLRBIT(FlagUseChkSum);                          /* no cs on 1st outbound  */
   CLRBIT(NET_ENABLE);                             /* disable transmitter    */

   switch (CmdMode) {
   default:                            /* oops...                            */
      CmdMode = 0;
   case 0 :                            /* interactive human mode             */
      SETBIT(FlagCharEcho);                        /* echo each character    */
      CLRBIT(FlagUseTE);                           /* don't enable trans     */
      ReportMode = REPORT_NONE;                    /* disable log outputs    */
      break;
   case 1 :                            /* network modes without feedback     */
   case 2 :
   case 3 :
      CLRBIT(FlagCharEcho);                        /* don't echo chars       */
      SETBIT(FlagUseTE);                           /* must enable/disable    */
      ReportMode = REPORT_NONE;                    /* disable log outputs    */
      break;
   }

}

/*---------------------------------------------------------------------------*/
/* Compute checksum on character buffer                                      */
/* First pointer is start of buffer, second is start of two-hexit checksum   */
/* Checksum is 8-bit addition, 2s complement, converted to hexits            */
/* Returned value is the calculated checksum, which should be zero if the    */
/*  incoming string had a proper checksum... otherwise, it is the checksum   */
/*  value inserted into the string                                           */

#define CS_TRACExxx

static char *Hexits = "0123456789ABCDEF";

int CmdCalcChkSum(pBuff,pCS)
char *pBuff;
char *pCS;
{
BYTE Sum;
char *pCSx;
char *pText;

   pText = pBuff;                      /* point to start of text...          */
   pCSx = pCS;                         /* set up pointer in XRAM             */
   Sum = GetHexits(&pCSx,2);           /* convert pair of hexits             */

#ifdef CS_TRACE
   printf("Starting checksum: string [%s] chars [%2s] value %02x\n",
          pBuff,pCS,Sum);
#endif

   memset(pCS,'0',2);                  /* replace CS chars with ASCII zeros  */

   while (*pText) {                    /* accumulate checksum                */
      Sum += *pText;
      ++pText;
   }

   Sum = -Sum;                         /* find 2s complement of result       */

   *pCS = Hexits[Sum >> 4];            /* insert new checksum chars          */
   *(pCS+1) = Hexits[Sum & 0x0F];

#ifdef CS_TRACE
   printf("Result is %2x:      string [%s]\n",Sum,pBuff);
#endif

   return 0x00ff & (int)Sum;           /* return calculated value            */

}



/*---------------------------------------------------------------------------*/
/* Send character to output with attention to network header prefix          */
/* Everything is copied into a checksum buffer until we hit the newline      */
/* Note that the checksum should NOT include the newline at the end...       */
/* Modified to allow \xff character to force line output                     */

Nputch(Ch)
int Ch;
{

   if (!CmdCSLength) {                             /* if first on line       */
      memset(CmdCSBuffer,0,sizeof(CmdCSBuffer));   /* flush checksum buffer  */
      if (CmdMode > 0) {
         *CmdCSBuffer = '$';                       /* our response trigger   */
         TESTBIT(TestBits,FlagUseChkSum);          /* need checksum?         */
         strcat(CmdCSBuffer,TestBits ? "00 " : " ");
         strcat(CmdCSBuffer,CmdNetAddress);        /* tack on addr + blank   */
      }
      CmdCSLength = strlen(CmdCSBuffer);           /* and record total len   */
   }


   if (('\n' == Ch) || ('\xff' == Ch)) {           /* end of line?           */
      TESTBIT(TestBits,FlagUseChkSum);
      if (TestBits) {
         CmdCalcChkSum(CmdCSBuffer,CmdCSBuffer+1); /* compute & install sum  */
      }
      putstr(CmdCSBuffer);                         /* send CS buffer         */
      if ('\n' == Ch) {
         putch('\n');                              /*  with this if needed   */
      }
      CmdCSLength = 0;                             /* set for next line      */
   }
   else {
      *(CmdCSBuffer + CmdCSLength) = Ch;           /* tack on the char       */
      ++CmdCSLength;                               /* account for it         */
   }

}


/*---------------------------------------------------------------------------*/
/* Send string to output with attention to net stuff                         */
/* You can use this with CmdRespBuff to simulate printfs                     */

Nputstr(pStr)
char *pStr;
{

   while (*pStr) {
      Nputch(*pStr);
      ++pStr;
   }

}

/*---------------------------------------------------------------------------*/
/* Verify that we've got a well-formatted command in the buffer              */
/* In mode 2 we return an error for bad checksum...                          */
/*  this obviously can't be used in a "real" network as there is no address  */
/* Returns pointer to start of command text or NULL if error                 */

unsigned CmdChkFormat(pBuffer)
char *pBuffer;
{
char *pText;
char *RetVal;
BYTE  State;
BYTE  ChkSum;

   RetVal = NULL;                      /* assume the worst                   */

   pText = pBuffer;
   State = FMT_TRIGGER;
   while (State < FMT_COMMAND) {       /* until cmd found or error           */

      switch (State) {
      case FMT_TRIGGER :               /* first character on the line        */
         CLRBIT(FlagUseChkSum);                    /* ensure this is off     */
         switch (*(pText++)) {                     /* note the increment     */
         case '!' :                                /* no checksum            */
            if (' ' == *pText) {                   /* so must have blank     */
               State = FMT_ADDRESS;
            }
            else {
               State = FMT_DISCARD;
            }
            break;
         case '#' :                                /* checksum on this line  */
            ChkSum = CmdCalcChkSum(pBuffer,pBuffer+1);      /* verify it     */
            if (ChkSum) {
               State = FMT_DISCARD;
               CSErrCount = min(29999,++CSErrCount);        /* tick counter  */
               if (2 == CmdMode) {
                  printf("*** Bad checksum: %2x\n",ChkSum);
               }
            }
            else {
               pText += 2;
               if (' ' == *pText) {
                  State = FMT_ADDRESS;
                  SETBIT(FlagUseChkSum);           /* reply must use sum     */
               }
               else {
                  State = FMT_DISCARD;
               }

            }
            break;
         default :                                 /* invalid line start     */
            State = FMT_DISCARD;
         }
         break;
      case FMT_ADDRESS :               /* network address                    */
         if (strncmp(CmdNetAddress,pText,strlen(CmdNetAddress))) {
            State = FMT_DISCARD;
         }
         else {
            State = FMT_COMMAND;
            RetVal = pText + strlen(CmdNetAddress);
         }
         break;

      }

      ++pText;                         /* step to next char                  */
   }

#ifdef CMD_TRACE
   if (!RetVal) {
      printf("[Format bad at: %s]\n",pText-1);
   }
#endif

   return RetVal;

}


/*---------------------------------------------------------------------------*/
/* Assemble complete command in buffer                                       */
/* Returns NULL until a \n shows up, then returns address of buffer          */
/* This allows it to be called from the main loop while we do other things   */
/*   ... a crude form of multitasking, but it serves our purposes here       */
/* The command string returned is terminated in 0 rather than a newline      */
/* If headers and checksums are enabled, we've got to get a clean line       */

char *CmdGetLine() {

char NewChar;
char *RetVal;

   RetVal = NULL;                      /* assume middle of line or bad CS    */

   if (!chkch()) {                     /* if no character, bail out          */
      goto Done;
   }

   NewChar = getch();                  /* fetch the character                */

#ifdef CMD_TRACE
   printf("[%02x %c %s]\n",
           NewChar,
           isprint(NewChar) ? NewChar : '-',
           CmdBuffer);
#endif

   TESTBIT(TestBits,FlagCharEcho);     /* echo it if enabled                 */
   if (TestBits) {
      putch(NewChar);
   }

/*--- add the character to the buffer                                        */
/*    we allow a little editing, but it makes no sense in network mode       */
/*     because there is no character echo...                                 */

   if (!CmdLineLen) {                              /* if first char...       */
      memset(CmdBuffer,0,sizeof(CmdBuffer));       /* clear out buffer       */
   }

   switch (NewChar) {
   case BS :
      if (CmdLineLen) {
         --CmdLineLen;                             /* back up one slot       */
         CmdBuffer[CmdLineLen] = 0;                /* discard that char      */
      }
      break;
   case '\n' :                                     /* do not add to buffer   */
      break;
   default :
      *(CmdBuffer + CmdLineLen) = NewChar;         /* add to buffer          */
      ++CmdLineLen;
      if (MAXCMDLEN <= CmdLineLen) {               /* trim any overflow      */
         CmdLineLen = MAXCMDLEN - 1;
      }
   }

/*--- when we hit the end of the line...                                     */

   if ('\n' == NewChar) {

      switch (CmdMode) {
      case 0 :                   /* interactive human mode                   */
         if ('*' != *CmdBuffer) {      /* echo from N2 -> N0 transition      */
            RetVal = CmdBuffer;        /* otherwise, good to go              */
         }
         break;

      case 1 :                   /* network mode w/o command echo            */
         RetVal = CmdChkFormat(CmdBuffer);         /* verify format          */
         break;

      case 2 :                   /* network mode with all commands echoed    */
         if (CmdLineLen) {             /* must have something to do...       */
            switch (*CmdBuffer) {
            case '$' :                 /* echoes from our commands           */
            case '*' :                 /* echoes from here                   */
            case '-' :                 /* diagnostic outputs                 */
               break;
            default :
               putstr("*");            /* flag these so we know them...      */
               putstr(CmdBuffer);      /* echo what we got                   */
               putstr("\n");           /* and add a newline at the end       */
               RetVal = CmdChkFormat(CmdBuffer);   /* verify format          */
            }
         }
         break;
      case 3 :                         /* must verify address & chksum first */
         if (CmdLineLen) {             /* must have something to do...       */
            switch (*CmdBuffer) {
            case '$' :                 /* echoes from our commands           */
            case '*' :                 /* echoes from here                   */
            case '-' :                 /* diagnostic outputs                 */
               break;
            default :
               RetVal = CmdChkFormat(CmdBuffer);   /* verify format          */
               if (RetVal) {           /* if line passes inspection...       */
                  putstr("*! ");       /* kill an echoed command echo        */
                  putstr(CmdNetAddress);           /* our address            */
                  putstr("OK\n");      /* seal of approval                   */
               }
            }
         }
         break;
      default :                  /* impossible to get here, punt it          */
         CmdNetMode(1);
      }

      CmdLineLen = 0;                  /* next char starts a new line        */
   }

Done:

   return RetVal;


}



/*---------------------------------------------------------------------------*/
/* Show messages only in human-on-line modes                                 */

CmdShowMsg(pMsg)
char *pMsg;
{

   if ((0 == CmdMode) || (2 == CmdMode)) {
      Nputstr(pMsg);
   }

}

/*---------------------------------------------------------------------------*/
/* Send out a prompt in plain interactive mode                               */
/* This bypasses the checksum buffer, for obvious reasons...                 */

CmdShowPrompt() {

   if (0 == CmdMode) {                 /* network modes?  0 = human...       */
      putstr(CmdNetAddress);           /* net address with a blank           */
      putstr(">");                     /* prompt character                   */
   }

}



/*---------------------------------------------------------------------------*/
/* Various command decoder routines                                          */
/* These are invoked from all the separate decoders to cut down on dupes     */

/*--- set the network address                                                */

CmdDecSetAddr(pCmd)
char *pCmd;
{

char *pChar;
BYTE Ctr;

   if ('=' == *(pCmd+1)) {
      pChar = pCmd+2;                              /* start of real address  */
      if (1 <= strlen(pChar)) {
         memset(CmdNetAddress,0,MAXADDRLEN+2);     /* set terminators        */
         for (Ctr=0;
              (Ctr<MAXADDRLEN) && *pChar;
              ++Ctr) {
            CmdNetAddress[Ctr] = *(pChar++);
         }
         *(CmdNetAddress + strlen(CmdNetAddress)) = ' ';    /* force blank   */
#ifdef COMMLINK2
         move(EEImage+EE_NETADDR,CmdNetAddress,MAXADDRLEN);
         CfgPutConfig();                           /* write it to EEPROM     */
#else
         CmdMakeAddress();                         /* update from port       */
#endif
         CmdSetAddrChk(CmdNetAddrChk);             /* make new checksum      */
      }
      else {
         CmdShowMsg("*** No address string...\n");
      }
   }
   else {
      CmdMakeAddress();                            /* update from port       */
      CmdSetAddrChk(CmdNetAddrChk);                /* recompute check value  */
      sprintf(CmdRespBuff,"Net address is %s\n",CmdNetAddress);
      Nputstr(CmdRespBuff);
   }

}


/*--- dump common internal status                                            */

CmdDecDumpState(pCmd)
char *pCmd;
{

   pCmd = pCmd;

   sprintf(CmdRespBuff,"Net address [%s]",CmdNetAddress);
   Nputstr(CmdRespBuff);

 asm {
      MOV   R0,#LOW(MAXSTACK+1)                    ; check stack from top
?slp
      DEC   R0
      MOV   A,[R0]
      JZ    ?slp
      MOV   TestBits,R0
 }

   sprintf(CmdRespBuff,".  Stack used %02x/%02x.\n",
          TestBits,MAXSTACK);
   Nputstr(CmdRespBuff);


}

/*--- dump error flags                                                       */

CmdDecDumpErrs(pCmd)
char *pCmd;
{

   pCmd = pCmd;

   sprintf(CmdRespBuff,"Error flags %02x.  Network checksum errors %u\n",
           ErrFlags,CSErrCount);
   Nputstr(CmdRespBuff);
   ErrFlags = 0;
   CSErrCount = 0;

}


/*--- set or report logging mode                                             */

CmdDecLogging(pCmd)
char *pCmd;
{

char *pChar;

   pChar = pCmd + 1;

   if (isdigit(*pChar)) {
      ReportMode = atoi(pChar);
   }
   else {
      sprintf(CmdRespBuff,"Logging mode is %u\n",ReportMode);
      Nputstr(CmdRespBuff);
   }

}

/*--- set or report network mode                                             */

CmdDecNetwork(pCmd)
char *pCmd;
{
char *pChar;

   pChar = pCmd + 1;

   switch (*pChar) {
   case 0 :
      sprintf(CmdRespBuff,"Net mode is %u\n",CmdMode);
      Nputstr(CmdRespBuff);
      break;
   default :
      if (isdigit(*pChar)) {
         CmdNetMode(atoi(pChar));
      }
      else {
         CmdShowMsg("*** N cmd syntax error\n");
      }
   }

}


/*--- report an invalid command                                              */

CmdDecNoCmd(pCmd)
char *pCmd;
{

   if (0 == CmdMode) {
      sprintf(CmdRespBuff,"*** Unknown command: [%s]\n",pCmd);
      CmdShowMsg(CmdRespBuff);
   }

}

