/* COMMAND.C -- command decoder                                              */

/*
#define CMD_TRACE
*/


/*---------------------------------------------------------------------------*/
/* Create network address from port bits                                     */
/* This is a dummy, as there can be only one PLLINK per network              */

#ifdef NETWORK

int CmdMakeAddress() {

   return 0;

}

#endif


/*---------------------------------------------------------------------------*/
/* Check for manual mode                                                     */

#ifdef NETWORK

int CmdCheckMode() {

asm {
   SETB  MANUALMODE                    ; ensure input on bit
   MOV   C,MANUALMODE                  ; pick up manual mode input
   CPL   C                             ; flip so jumper installed = 1
   CLR   A                             ; start with zero
   MOV   A.0,C                         ; combine with control bit
   MOV   TestBits,A                    ; set up for C code
}

   if (TestBits) {
      CmdNetMode(0);                   /* jumper installed sets manual mode  */
   }
   else {
      CmdNetMode(NETMODE);             /* all other addresses are automatic  */
   }

   return;

}

#endif


/*---------------------------------------------------------------------------*/
/* Decode commands                                                           */

CmdDecode(pCmd)
char *pCmd;
{
char *pChar;
char *pArg;
int ParseErr;

   pChar = pArg = pCmd;                /* uppercase & squash blanks          */
   while (*pChar) {
      if (' ' == *pChar) {                         /* skip blanks here       */
         ++pChar;
      }
      else {
         *pArg = toupper(*pChar);
         ++pChar;                                  /* copy the rest          */
         ++pArg;
      }
   }
   *pArg = 0;                          /* force terminator after squash      */
   pChar = pCmd+1;                     /* set up for arg if any              */

   switch (*pCmd) {
   case 'A' :                          /* dump or set network address        */
      CmdDecSetAddr(pCmd);
      break;
   case 'D' :                          /* dump internal status               */
      CmdDecDumpState();
      PwrDumpState();
      break;
   case 'E' :                          /* show error flags                   */
      CmdDecDumpErrs(pCmd);
      break;
   case 'F' :                          /* flush ring buffers                 */
      PwrFlushTrans();
      PwrFlushRec();
      break;
   case 'L' :                          /* set receiver logging modes         */
      CmdDecLogging(pCmd);
      break;
   case 'M' :                          /* set manual input mode              */
      if (isdigit(*pChar)) {
         RmtCtlMode = atoi(pChar);
      }
      else {
         sprintf(CmdRespBuff,"Manual remote input mode is %u\n",RmtCtlMode);
         Nputstr(CmdRespBuff);
      }
      break;
   case 'N' :                          /* select net mode                    */
      CmdDecNetwork(pCmd);
      break;
   case 'P' :                          /* get power fail status              */
      TESTBIT(TestBits,FlagPwrFail);
      sprintf(CmdRespBuff,"Fail=%u,",TestBits);                 /* current state          */
      Nputstr(CmdRespBuff);
      TESTBIT(TestBits,FlagAnyPFail);
      sprintf(CmdRespBuff,"%u\n",TestBits);                     /* any failures?          */
      Nputstr(CmdRespBuff);
      CLRBIT(FlagAnyPFail);                        /* reset memory           */
      break;
   case 'Q' :                          /* query X10 state                    */
      switch (*pChar) {
      case 0 :                                     /* no args, full dump     */
         PwrShowModule(0xFF,0);                    /* full dump              */
         break;
      case 'S' :                                   /* full hex dump          */
         PwrShowModule(0xFE,0x00);
         break;
      default :
         while (*pChar) {                          /* handle multiple parms  */
            ParseErr = CmdGetXArgs(&pChar);     /* module 00 becomes FF   */
            if ((!ParseErr) ||
                ((CMD_BADMOD == ParseErr) && (0xFF == CmdModuleNum)) ||
                (CMD_BADFUNC == ParseErr)) {
               PwrShowModule(CmdHouseNum,CmdModuleNum);
            }
            else {
               CmdShowMsg("*** Q cmd syntax error\n");
            }
         }
      }
      break;
   case 'R' :                          /* set refresh interval               */
      switch (*pChar) {
      case 0 :                                     /* no args, report time   */
         sprintf(CmdRespBuff,"Refresh cycle %u sec with %u modules\n",
                 RefPeriod,RefGetEvents());
         Nputstr(CmdRespBuff);
         break;
      case 'C' :                                   /* clear & set minutes    */
         RefClear(60*atoi(pChar+1));
         break;
      case 'E' :
         if (strcmp(pChar,"ESET")) {
            CmdShowMsg("*** Unknown command\n");
         }
         else {
            SerWaitOutDone();
 asm {
   LJMP 0                              ; solid reset...
 }
         }
         break;
      case 'Q' :                                   /* dump current events    */
         RefDumpTable();
         break;
      case 'S' :
         RefSet(atoi(pChar+1));                    /* no clear, in seconds   */
         break;
      default :
         if (isdigit(*pChar)) {
            RefSet(60*atoi(pChar));                /* no clear, in minutes   */
         }
         else {
            CmdShowMsg("*** R cmd syntax error\n");
         }
      }
      break;
   case 'S' :                          /* send X10 command                   */
      while (*pChar) {                             /* handle multiple parms  */
         ParseErr = CmdGetXArgs(&pChar);
         if (ParseErr) {
            CmdShowMsg("*** S cmd syntax error\n");
         }
         else {
            PwrSendSeq(CmdHouseNum,CmdModuleNum,CmdFuncNum,CmdRepeats);
            RefNewEvent(CmdHouseNum,CmdModuleNum,CmdFuncNum,CmdRepeats);
         }
      }
      break;
   case 0 :                            /* ignore empy line                   */
      break;
   case '?' :
      Nputstr("Available commands (values are decimal unless noted)\n");
      Nputstr(" A=string  +  set network address\n");
      Nputstr(" D         *  dump internal status variables (most in hex)\n");
      Nputstr(" E         *  dump and clear error flags\n");
      Nputstr(" F            flush trans/rec buffers\n");
      Nputstr(" L n          set reporting modes (bitmapped)\n");
      Nputstr(" M n          set manual remote control mode\n");
      Nputstr(" N n       +* set network mode\n");
      Nputstr(" P            display and clear power fail status\n");
      Nputstr(" Q            dump all modules\n");
      Nputstr(" Q h          dump all modules for housecode\n");
      Nputstr(" Q h mm       dump single module for housecode\n");
      Nputstr(" Q S          dump all modules in hex\n");
      Nputstr(" R            dump refresh interval\n");
      Nputstr(" R n          set refresh in minutes\n");
      Nputstr(" R C n        clear refresh buffer, set in minutes\n");
      Nputstr(" R S n        set refresh in seconds\n");
      Nputstr(" R Q          dump refresh table entries\n");
      Nputstr(" RESET        perform power-on reset initialization\n");
      Nputstr(" S h mm cc rr send command to module at housecode\n");
      Nputstr("Modules must be specified with two digits:  S N 07 ON\n");
      Nputstr("+ marks network-only commands\n");
      Nputstr("* marks debugging commands\n");
      break;
   default :
      CmdDecNoCmd(pCmd);
   }

   return;


}


/*---------------------------------------------------------------------------*/
/* Extract house, device, function, and steps from the command line          */
/* Result is in global variables                                             */
/* Returns 0 if OK, 1 if error                                               */
/* ppArgs comes back pointing to the next argument or the string terminator  */

int CmdGetXArgs(ppArgs)
char **ppArgs;
{
int State;
char *pArgs;
int RetCode;

   RetCode = 0;                        /* assume failure                     */
   State = 0;
   pArgs = *ppArgs;                    /* avoid double indirection           */

   CmdRepeats = 2;                     /* default if not set in command      */
   CmdModuleNum = 0xFF;                /* preset to fix missing module       */

   while ((*pArgs) && (!RetCode) && (4 > State)) {

      switch (State) {
      case 0 :                                     /* extract house code     */
         if (('A' <= *pArgs) && ('P' >= *pArgs)) {
            CmdHouseNum = *pArgs - 'A';
            ++pArgs;
            ++State;
         }
         else {
            RetCode = CMD_BADHOUSE;
         }
         break;

      case 1 :                                     /* extract device code    */
         if ((0 <= (CmdModuleNum = PwrFindFunc(pArgs))) &&
             (16 > CmdModuleNum)) {
            pArgs += 2;
            ++State;
         }
         else {
            RetCode = CMD_BADMOD;
         }
         break;

      case 2 :                                     /* extract function code  */
         CmdFuncNum = PwrFindFunc(pArgs);
#ifdef CMD_TRACE
         printf("Returned value is %02x...",CmdFuncNum);
#endif
         if ((0xFF != CmdFuncNum) && (16 <= CmdFuncNum)) {
            pArgs += 2;
            ++State;
         }
         else {
            RetCode = CMD_BADFUNC;
         }
         break;

      case 3 :                                     /* extract repeat count   */
         if (isdigit(*pArgs)) {
            CmdRepeats = atoi(pArgs);
            while (isdigit(*pArgs)) {              /* step over all digits   */
               ++pArgs;
            }
         }
         else {
            CmdRepeats = 2;                        /* if bad spec in cmd     */
         }
         ++State;
         break;
      }
   }

   while (*pArgs && (',' != *pArgs)) {             /* skip to next arg       */
      ++pArgs;
   }
   if (',' == *pArgs) {
      ++pArgs;
   }

   *ppArgs = pArgs;                    /* update output pointer              */

   return RetCode;

}

