/* Home Control System II                                                    */
/* Smart X-10 controller using TW-523 power-line interface module            */
/* Copyright 1992 Circuit Cellar Inc                                         */
/* Firmware Furnace #25  Ed Nisley                                           */

/* Compiled with Micro-C using Compact model                                 */
/* External RAM must be in Code/Data space                                   */
/* Local variables are in external RAM                                       */

/* See READ.ME for details on changes to default Micro-C files               */

#include <8051reg.h>
#include <8051io.h>

#include "e:\mc\cust51\8051base.h"
#include "e:\mc\cust51\8051sfb.h"

/*---------------------------------------------------------------------------*/
/* Constants                                                                 */

#define VERSION "1.2d"

/*--- define NETWORK to produce RS-485 version of the code                   */
/*    define NONCOMMERCIAL to produce BBS version of distribution code       */
/*    these are normally contained in the build.ctl code and go like this:   */
/*  #define NETWORK          or NETWORKxxx                                   */
/*  #define NONCOMMERCIAL    or NONCOMMERCIALxxx                             */

#include "build.ctl"

#include "pllink.h"

#ifdef NETWORK
#include "..\network\console.h"
#include "..\network\command.h"
#else
#include "..\direct\console.h"
#include "..\direct\command.h"
#endif

#include "command.h"

#include "acline.h"
#include "refresh.h"

/*---------------------------------------------------------------------------*/
/* Global variables                                                          */

/*--- these start at 0008 as defined in 8051rlp1.asm                         */
/*    0008-9 is the External RAM stack pointer used by the compiler          */

NEAR BYTE ErrFlags;

#ifdef NETWORK
#include "..\network\console.d1"
#else
#include "..\direct\console.d1"
#endif

#include "acline.d1"


/*--- bit variables must be located at IRAM addresses 0x20 through 0x3F      */
/*    note that bit variables from other files must be accounted for...      */

NEAR BYTE Padding[0x1F - AC_D1 - CON_D1 - 0x08 - 3];
NEAR BYTE TestBits;                    /* 1F global temp for TESTBIT macro   */

#ifdef NETWORK
#include "..\network\console.b"
#include "..\network\command.b"
#else
#include "..\direct\console.b"
#include "..\direct\command.b"
#endif

#include "acline.b"

/*---------------------------------------------------------------------------*/
/* Variables in RAM and constants in EPROM                                   */
/* All initialized "variables" are located in EPROM...                       */

#ifdef NETWORK
#include "..\network\console.d2"
#include "..\network\command.d2"
#else
#include "..\direct\console.d2"
#include "..\direct\command.d2"
#endif

#include "command.d2"
#include "acline.d2"
#include "refresh.d2"

char CmdDefAddr[] = "X10 ";                        /* default net address    */

BYTE ReportMode;


/*---------------------------------------------------------------------------*/
/* Initializations                                                           */

Initialize() {

   ErrFlags = 0;                       /* clear error monitors               */

 asm {
   MOV   R0,SP                         ; clear the stack for bounds check
   INC   R0
   CLR   A
?ilp
   MOV   [R0],A
   INC   R0
   CJNE  R0,#LOW(MAXSTACK+1),?ilp      ; can wrap from FF to 100 just fine
 }


}


/*---------------------------------------------------------------------------*/
/* Code for the various functions                                            */

#ifdef NETWORK
#include "..\network\console.c"
#include "..\network\command.c"
#else
#include "..\direct\console.c"
#include "..\direct\command.c"
#endif

#include "command.c"
#include "acline.c"
#include "refresh.c"

/*---------------------------------------------------------------------------*/
/* The Main Event                                                            */

main() {

WORD CmdAddr;
WORD XInMsg;

   Initialize();

   PwrInitialize();
   SerInitialize();

   SETBIT(EA);                   /* start up interrupt handlers              */

   CmdInitialize();              /* set up command decoder                   */
   RefInitialize();              /* start up refreshing code                 */

#ifdef NETWORK
   if (!CmdMode) {
      Nputstr("\nPL-Link Networked X10 Controller, Version ");
      Nputstr(VERSION);
      Nputstr(" for ");
      Nputstr(FREQMSG);
      Nputstr("\n  Copyright 1992 Circuit Cellar, Inc.\n");
#ifdef NONCOMMERCIAL
      Nputstr("  NOT FOR COMMERCIAL USE\n");
/*    Nputstr("  Special Debugging Version!\n");  */
      Nputstr("  Firmware Furnace #25,  Ed Nisley\n");
#endif
   }
#else
   Nputstr("\nSmart X10 Controller, Version ");
   Nputstr(VERSION);
   Nputstr(" for ");
   Nputstr(FREQMSG);
   Nputstr("\n  Copyright 1992 Circuit Cellar, Inc.\n");
#ifdef NONCOMMERCIAL
   Nputstr("  NOT FOR COMMERCIAL USE\n");
/* Nputstr("  Special Debugging Version!\n");  */
   Nputstr("  Firmware Furnace #25,  Ed Nisley\n");
#endif
#endif

   CmdShowPrompt();              /* show prompt if in human mode...          */

   while (TRUE) {

      TESTBIT(TestBits,FlagSecond);    /* another second gone by?            */
      if (TestBits) {
         CPLBIT(MAINFLAG);             /* visible heartbeat & sync           */
         CPLBIT(MAINFLAG);             /* this allows sync on both edges     */
         CPLBIT(MAINFLAG);
         CLRBIT(FlagSecond);
         RefCheck();                   /* check for X10 refresh action       */
      }

      while (XMSG_NOMSG != (XInMsg = PwrGetMsg())) {        /* do receiver   */
         if (0x01 == (RmtCtlMode & 0x01)) {        /* only if enabled!       */
            PwrUpdateState(XInMsg);
         }
      }

      if (NULL != (CmdAddr = CmdGetLine())) {      /* any commands?          */
         CmdDecode(CmdAddr);                       /* and decode them        */
         CmdShowPrompt();                          /* prompt for humans      */
      }

   }

   Nputstr("--- Fell out the bottom!\n");           /* impossible...          */

}

