/* COMMAND.C -- command decoder                                              */



/*---------------------------------------------------------------------------*/
/* Create network address from P1 bits                                       */
/* The next-to-last character of the string gets the address digit           */
/*  (the string should always end in a blank, but we do not check for this)  */

#ifdef NETWORK

int CmdMakeAddress() {

BYTE AddrLen;

   AddrLen = strlen(CmdNetAddress);    /* figure out target character        */

   if (AddrLen < 2) {                  /* bad address, fix it up first       */
      move(CmdNetAddress,CmdDefAddr,MAXADDRLEN+1);
      AddrLen = strlen(CmdNetAddress);
   }

   TestBits = CmdNetAddress[AddrLen - 2];          /* set up char for mods   */

asm {
   MOV   A,TestBits                    ; pick up address char
   SETB  P3.2                          ; allow input on bits
   SETB  P3.3
   MOV   C,P3.3                        ; pick up the bits
   MOV   A.1,C
   MOV   C,P3.2
   MOV   A.0,C
   XRL   A,#$03                        ; make jumper installed = 1
   MOV   TestBits,A                    ; set up for C code
}

   CmdNetAddress[AddrLen - 2] = TestBits;          /* install new bits       */

   return;

}

#endif

/*---------------------------------------------------------------------------*/
/* Check for manual mode                                                     */

#ifdef NETWORK

int CmdCheckMode() {

asm {
   SETB  P3.2                          ; allow input on bits
   SETB  P3.3
   CLR   A                             ; default is address 0
   MOV   C,P3.3                        ; pick up the bits
   MOV   A.1,C
   MOV   C,P3.2
   MOV   A.0,C
   XRL   A,#$03                        ; make jumper installed = 1
   MOV   TestBits,A                    ; set up for C code
}

   if (3 == TestBits) {
      CmdNetMode(0);                   /* address 3 sets manual mode         */
   }
   else {
      CmdNetMode(NETMODE);             /* all other addresses are automatic  */
   }

   return;

}

#endif


/*---------------------------------------------------------------------------*/
/* Decode commands                                                           */

CmdDecode(pCmd)
char *pCmd;
{
char *pChar;
char *pArg;
BYTE TempByte;

   pChar = pArg = pCmd;                /* uppercase & squash blanks          */
   while (*pChar) {
      if (' ' == *pChar) {                         /* skip blanks here       */
         ++pChar;
      }
      else {
         *pArg = toupper(*pChar);
         ++pChar;                                  /* copy the rest          */
         ++pArg;
      }
   }
   *pArg = 0;                          /* force terminator after squash      */
   pChar = pCmd+1;                     /* set up for arg if any              */

   switch (*pCmd) {
   case 'A' :                          /* dump or set network address        */
      CmdDecSetAddr(pCmd);
      break;
   case 'D' :                          /* dump internal or stored buffers    */
      CmdDecDumpState();
      sprintf(CmdRespBuff,"T0Ticks %u\n",T0Ticks);
      Nputstr(CmdRespBuff);
      break;
   case 'E' :                          /* show error flags                   */
      CmdDecDumpErrs(pCmd);
      sprintf(CmdRespBuff," EPROM test: %04x  RAM test: %04x\n",
              ROM_ChkSum,RAM_FailAddr);
      Nputstr(CmdRespBuff);
      break;
   case 'L' :                          /* set debug logging options          */
      CmdDecLogging(pCmd);
      break;
   case 'N' :                          /* select net mode                    */
      CmdDecNetwork(pCmd);
      break;
   case 'Q' :                          /* query input values                 */
      while (*pChar) {
         if (IOFormatData(&pChar)) {   /* and display them for the public    */
            break;
         }
         if (',' == *pChar) {          /* step over separator                */
            ++pChar;
            Nputch(',');                           /* add one to output      */
         }
         TmrHeartBeat();
      }
      Nputch('\n');
      break;
   case 'R' :
      if (strcmp(pChar,"ESET")) {                  /* nonzero = no match     */
         if (isdigit(*pChar)) {
            TempByte = atoi(pChar) & 0x00FF;
            if (TempByte < NUM_TIMES) {
              SartDly = SartTimes[TempByte];
              sprintf(CmdRespBuff,"Rate selector is %u\n",TempByte);
              Nputstr(CmdRespBuff);
            }
            else {
               sprintf(CmdRespBuff,"Invalid bit rate selector: %u\n",
                       TempByte);
               Nputstr(CmdRespBuff);
            }
         }
          else {
             TempByte=0;
             while ((TempByte < NUM_TIMES) &&
                    (SartTimes[TempByte] != SartDly)) {
               ++TempByte;
             }
             sprintf(CmdRespBuff,"S-ART bit rate selector is %u\n",TempByte);
             Nputstr(CmdRespBuff);
          }

      }
      else {
         SerWaitOutDone();
 asm {
   LJMP   0                            ; solid reset...
 }
      }
      break;
   case 'S' :                          /* set output value                   */
      while (*pChar) {
         if (IOSetValue(&pChar)) {
            break;
         }
         if (',' == *pChar) {          /* step over separator                */
            ++pChar;
         }
         TmrHeartBeat();
      }
      break;
   case 'T' :
      if (*pChar) {
         if (isdigit(*pChar)) {
            SartTestMode = atoi(pChar) & 0x00FF;
            sprintf(CmdRespBuff,"Entering S-ART network test mode %u\n",
                    SartTestMode);
            Nputstr(CmdRespBuff);
         }
         else {
               sprintf(CmdRespBuff,
                       "Need a number to select S-ART test mode\n");
               Nputstr(CmdRespBuff);
         }
      }
      else {
         sprintf(CmdRespBuff,"S-ART network test mode is %u\n",SartTestMode);
         Nputstr(CmdRespBuff);
      }
      break;
   case 0 :                            /* ignore empy line                   */
      break;
   case '?' :
      TmrHeartBeat();
      Nputstr("Available commands (values in decimal unless noted)\n");
      Nputstr(" A=string   + set network address to string\n");
      Nputstr(" D          * display internal status variables (some in hex)\n");
      Nputstr(" E          * show and clear error flags\n");
      Nputstr(" L n          set reporting mode (bitmapped)\n");
      Nputstr(" N n       *+ set network mode\n");
      Nputstr(" Q            query network inputs\n");
      TmrHeartBeat();
      Nputstr(" RESET        perform power-on reset initialization\n");
      Nputstr(" R n          set S-ART network bit rate\n");
      Nputstr(" S            set network output values\n");
      Nputstr(" T          * activate S-ART test modes\n");
      Nputstr("+ marks network-only commands\n");
      Nputstr("* marks debugging commands\n");
      TmrHeartBeat();
      break;
   default :
      CmdDecNoCmd(pCmd);
   }

   return;

}

