/* NETIO.C                                                                   */

#define TRC_IO


/*---------------------------------------------------------------------------*/
/* Set up I/O devices and initialize the arrays                              */

IOInitialize() {

/*--- global variables                                                       */

   memset(SartData,                    /* preset output bits HIGH            */
          MASK_OUT1 | MASK_OUT0,
          sizeof(SartData));
   SartData[0][30] = 0;                /* special-case the extra bytes       */
   SartData[0][31] = 0;
   SartData[1][30] = 0;
   SartData[1][31] = 0;

   NetIOBits = 0;                      /* flush all the flags                */
   SartAddr = -1;                      /* force resync on first pass         */
   SartDly = SartTimes[0];             /* default to slowest time            */
   SartTestMode = SartTestPass = 0;    /* disable testing stuff              */

/*--- turn off the network output controls                                   */
/*    these will be set up during the first scan, but this feels good        */

   SETBIT(A_DIS_HIGH);
   SETBIT(A_DIS_MID);
   CLRBIT(A_EN_LOW);                   /* this is disabled, too              */
   SETBIT(A_SENSE);                    /* allow input on this line           */

   SETBIT(B_DIS_HIGH);
   SETBIT(B_DIS_MID);
   CLRBIT(B_EN_LOW);                   /* this is disabled, too              */
   SETBIT(B_SENSE);                    /* allow input on this line           */


}


/*---------------------------------------------------------------------------*/
/* Extract function, net ID, SART address, and bit ID from command string    */
/* Argument is pointer to pointer to characters                              */
/* Always returns char ptr incremented past last char used                   */
/* Return value is function index or NOFUNC if invalid                       */

int IOParse(ppSpec)
char **ppSpec;
{
WORD RetVal;
BYTE FnIndex;
char *pBuffer;
BYTE Temp;

   RetVal = NOFUNC;

   NetFunction = NOFUNC;               /* assume nothing is specified        */
   NetID = NONET;
   NetAddr = NOADDR;
   NetBit = NOBIT;

/*--- extract chars up to the first non-alphanumeric char                    */
/*    then look it up in the table for a match                               */

   memset(IOParseBuff,0,sizeof(IOParseBuff));
   pBuffer = IOParseBuff;
   while (isascii(**ppSpec) && isalnum(**ppSpec)) {         /* force caps    */
      *(pBuffer++) = toupper(**ppSpec);
      ++(*ppSpec);
   }

   for (FnIndex=0; FnIndex<NUMFUNCS; ++FnIndex) {
      if (!strcmp(IOParseBuff,FnName[FnIndex])) {
         break;
      }
   }

   if (NUMFUNCS == FnIndex) {
      sprintf(CmdRespBuff,"*** Bad function [%s]\n",IOParseBuff);
      CmdShowMsg(CmdRespBuff);
      goto Done;
   }
   else {
      RetVal = FnIndex;                /* we have a good return code         */
      NetFunction = FnIndex;
      if (REPORT_DEC & ReportMode) {
         sprintf(CmdRespBuff,"[Function %s index %d",IOParseBuff,NetFunction);
         Nputstr(CmdRespBuff);
      }
   }


/*--- pluck the network ID from the function name                            */
/*    this depends on the last character being either A or B...              */

   NetID = IOParseBuff[strlen(IOParseBuff) - 1] - 'A';
   if (REPORT_DEC & ReportMode) {
      sprintf(CmdRespBuff,", Net ID %d",NetID);
      Nputstr(CmdRespBuff);
   }


/*--- extract SART address from the command string if we are at a dot        */

   if ('.' == **ppSpec) {
      ++(*ppSpec);
      Temp = GetDecimal(ppSpec);
      if (Temp < MAX_SARTS) {
         NetAddr = Temp;
         if (REPORT_DEC & ReportMode) {
            sprintf(CmdRespBuff,", address = %u",Temp);
            Nputstr(CmdRespBuff);
         }
      }
      else {
         sprintf(CmdRespBuff,"\n*** Bad address: %u\n",Temp);
         CmdShowMsg(CmdRespBuff);
         goto Done;
      }
   }
   else {
      if (REPORT_DEC & ReportMode) {
         Nputstr(", all addresses");
      }
   }

/*--- extract bit spec if we are once again at a dot                         */

   if ('.' == **ppSpec) {
      ++(*ppSpec);
      Temp = GetDecimal(ppSpec);
      if (Temp < 2) {
         NetBit = Temp;
         if (REPORT_DEC & ReportMode) {
            sprintf(CmdRespBuff,", bit = %u]\n",Temp);
            Nputstr(CmdRespBuff);
         }
      }
      else {
         sprintf(CmdRespBuff,"\n*** Bad bit ID: %u\n",Temp);
         CmdShowMsg(CmdRespBuff);
         goto Done;
      }
   }
   else {
      if (REPORT_DEC & ReportMode) {
         Nputstr(", all bits]\n");
      }
   }

/*--- return the function code, which may be NOFUNC if no match              */

Done:
   return RetVal;

}



/*---------------------------------------------------------------------------*/
/* Set output bit (or bits!) from command string                             */
/* Returns string pointer ticked to just beyond the value... if any          */
/* Function return value is 0 for OK, 1 for error                            */

int IOSetValue(ppCmd)
char **ppCmd;
{
int RetCode;
BYTE Index;
WORD Value;
BYTE *pEntry;

   RetCode = 1;                                    /* assume failure         */

   if (NOFUNC == IOParse(ppCmd)) {                 /* find targets           */
      goto Done;
   }

   if (('=' != **ppCmd) || !(*((*ppCmd)+1))) {     /* check for =data        */
      CmdShowMsg("*** Set command missing data\n");
      goto Done;
   }

   ++(*ppCmd);                                     /* step over = sign       */

   if (REPORT_SET & ReportMode) {
      sprintf(CmdRespBuff,"[Net Function %d",NetFunction);
      Nputstr(CmdRespBuff);
   }

   switch (NetFunction) {
   case FN_NETA :                      /* set output bit (or all of them!)   */
   case FN_NETB :
      Value = GetDecimal(ppCmd);                   /* should be one digit    */
      if (REPORT_SET & ReportMode) {
         sprintf(CmdRespBuff,", NetID is %d, Addr is %02x, Data is %d]\n",
                NetID,NetAddr,Value);
         Nputstr(CmdRespBuff);
      }
      if (NOADDR == NetAddr) {                     /* apply to whole net?    */
         if (Value < 4) {
            for (Index=0; Index < MAX_SARTS; ++Index) {
               pEntry = &(SartData[NetID][Index]);
               *pEntry &= ~(MASK_OUT1 | MASK_OUT0);
               *pEntry |= (Value & 0x01) ? MASK_OUT0 : 0;
               *pEntry |= (Value & 0x02) ? MASK_OUT1 : 0;
            }
            RetCode = 0;
         }
         else {
            CmdShowMsg("*** Value must be between 0 and 3\n");
            goto Done;
         }
      }
      else {                                       /* apply to single addr   */
         if (NOBIT == NetBit) {                    /* apply to both bits     */
            if (Value < 4) {
               pEntry = &(SartData[NetID][NetAddr]);
               *pEntry  &= ~(MASK_OUT1 | MASK_OUT0);
               *pEntry |= (Value & 0x01) ? MASK_OUT0 : 0;
               *pEntry |= (Value & 0x02) ? MASK_OUT1 : 0;
               RetCode = 0;
            }
            else {
               CmdShowMsg("*** Value must be between 0 and 3\n");
               goto Done;
            }
         }
         else {                                    /* apply to given bit     */
            if (Value < 2) {
               switch (NetBit) {
               case 0 :
                  SartData[NetID][NetAddr] &= ~MASK_OUT0;
                  SartData[NetID][NetAddr] |= Value ? MASK_OUT0 : 0;
                  break;
               case 1 :
                  SartData[NetID][NetAddr] &= ~MASK_OUT1;
                  SartData[NetID][NetAddr] |= Value ? MASK_OUT1 : 0;
                  break;
               }
               RetCode = 0;
            }
            else {
               CmdShowMsg("*** Value must be either 0 or 1\n");
               goto Done;
            }
         }
      }
      break;
   case FN_OUTA :
   case FN_OUTB :
      if ((NOADDR == NetAddr) && (NOBIT == NetBit)) {
         memset(IOParseBuff,0,sizeof(IOParseBuff));
         for (Index=0; Index < 32; Index+=2) {     /* split hexits in twain  */
            if (!isxdigit(**ppCmd)) {
               break;
            }
            Value = GetHexits(ppCmd,1);
            IOParseBuff[Index] = Value >> 2;       /* high net address       */
            IOParseBuff[Index+1] = Value & 0x03;   /* low net address        */
         }
         if (!Index) {
            CmdShowMsg("*** No data values\n");
            goto Done;
         }
         --Index;                                  /* back up to last byte   */
         for (NetAddr=0; NetAddr<MAX_SARTS; ++NetAddr) {    /* set outputs   */
            pEntry = &(SartData[NetID][NetAddr]);
            *pEntry &= ~(MASK_OUT1 | MASK_OUT0);
            *pEntry |= (IOParseBuff[Index] & 0x01) ? MASK_OUT0 : 0;
            *pEntry |= (IOParseBuff[Index] & 0x02) ? MASK_OUT1 : 0;
            if (!Index) {                          /* bail out at start...   */
               break;
            }
            --Index;
         }
         RetCode = 0;
      }
      else {
         CmdShowMsg("*** Cannot use address or bit ID with this function\n");
      }
      break;
   case FN_INA :
   case FN_INB :
      CmdShowMsg("*** Cannot set input bits\n");
      break;
   case FN_STATA :
   case FN_STATB :
      CmdShowMsg("*** Cannot set status bits\n");
      break;
   }

Done:

   return RetCode;

}




/*---------------------------------------------------------------------------*/
/* Displays current state of IN, OUT, or status bits                         */
/* Produces appropriate text to identify the results                         */

int IOFormatData(ppCmd)
char **ppCmd;
{
int RetCode;
BYTE FailCode;
BYTE Value,Entry;

   RetCode = 1;

   if (NOFUNC == IOParse(ppCmd)) {                 /* find targets           */
      goto Done;
   }

/*--- set up the introduction...                                             */

   strcpy(IOParseBuff,FnName[NetFunction]);        /* set up device name     */

   if (!(NOADDR == NetAddr)) {                     /* add address & bit      */
      sprintf(IOParseBuff+strlen(IOParseBuff),".%u",NetAddr);
      if (!(NOBIT == NetBit)) {
         sprintf(IOParseBuff+strlen(IOParseBuff),".%u",NetBit);
      }
   }

   strcat(IOParseBuff,"=");

/*--- now format the data                                                    */

   if (REPORT_GET & ReportMode) {
      sprintf(CmdRespBuff,"[Net Function %d",NetFunction);
      Nputstr(CmdRespBuff);
   }

   switch (NetID) {                    /* set up network failure code        */
   case 0 :
      TESTBIT(TestBits,FlagNetAFail);
      break;
   case 1 :
      TESTBIT(TestBits,FlagNetBFail);
      break;
   }
   FailCode = TestBits;

   switch (NetFunction) {
   case FN_NETA :                      /* set output bit (or all of them!)   */
   case FN_NETB :
      if (REPORT_GET & ReportMode) {
         sprintf(CmdRespBuff,", NetID %d, Addr %02x\n",
                NetID,NetAddr);
         Nputstr(CmdRespBuff);
      }

      if (NOADDR == NetAddr) {                     /* must specify address   */
         CmdShowMsg("*** Must specify address\n");
      }
      else {                                       /* apply to single addr   */
         Entry = SartData[NetID][NetAddr];         /* set up data value      */
         if (REPORT_GET & ReportMode) {
            sprintf(CmdRespBuff,", Data %02x]\n",Entry);
            Nputstr(CmdRespBuff);
         }
         if (NOBIT == NetBit) {                    /* want both I/O bits     */
            sprintf(IOParseBuff+strlen(IOParseBuff),
                  "%d%d|%d%d|%d",
                  (Entry & MASK_OUT1) ? 1 : 0,
                  (Entry & MASK_OUT0) ? 1 : 0,
                  (Entry & MASK_IN1) ? 1 : 0,
                  (Entry & MASK_IN0) ? 1 : 0,
                  (FailCode << 1) + (Entry & MASK_PTYERR) ? 1 : 0);
            RetCode = 0;
         }
         else {                                    /* apply to given bit     */
            sprintf(IOParseBuff+strlen(IOParseBuff),
                  "%d|%d|%d",
                  (Entry & (NetBit ? MASK_OUT1 : MASK_OUT0)) ? 1 : 0,
                  (Entry & (NetBit ? MASK_IN1 : MASK_IN0)) ? 1 : 0,
                  (FailCode << 1) + (Entry & MASK_PTYERR) ? 1 : 0);
            RetCode = 0;
         }
      }
      break;
   case FN_OUTA :
   case FN_OUTB :
      if ((NOADDR == NetAddr) && (NOBIT == NetBit)) {
         for (NetAddr=MAX_SDATA-1; NetAddr<MAX_SDATA; NetAddr-=2) {
            Value  = (SartData[NetID][NetAddr] &  MASK_OUT1) ? 0x08 : 0;
            Value |= (SartData[NetID][NetAddr] &  MASK_OUT0) ? 0x04 : 0;
            Value |= (SartData[NetID][NetAddr-1] &  MASK_OUT1) ? 0x02 : 0;
            Value |= (SartData[NetID][NetAddr-1] &  MASK_OUT0) ? 0x01 : 0;
            sprintf(IOParseBuff+strlen(IOParseBuff),"%01x",Value);
         }
         RetCode = 0;
      }
      else {
         CmdShowMsg("*** Cannot use address or bit ID with this function\n");
      }
      break;
   case FN_INA :
   case FN_INB :
      if ((NOADDR == NetAddr) && (NOBIT == NetBit)) {
         for (NetAddr=MAX_SDATA-1; NetAddr<MAX_SDATA; NetAddr-=2) {
            Value  = (SartData[NetID][NetAddr] &  MASK_IN1) ? 0x08 : 0;
            Value |= (SartData[NetID][NetAddr] &  MASK_IN0) ? 0x04 : 0;
            Value |= (SartData[NetID][NetAddr-1] &  MASK_IN1) ? 0x02 : 0;
            Value |= (SartData[NetID][NetAddr-1] &  MASK_IN0) ? 0x01 : 0;
            sprintf(IOParseBuff+strlen(IOParseBuff),"%01x",Value);
         }
         RetCode = 0;
      }
      else {
         CmdShowMsg("*** Cannot use address or bit ID with this function\n");
      }
      break;
   case FN_STATA :
   case FN_STATB :
      if ((NOADDR == NetAddr) && (NOBIT == NetBit)) {
         for (NetAddr=MAX_SDATA-1; NetAddr<MAX_SDATA; NetAddr-=2) {
            Value  = FailCode * 0x0a;              /* set up two flags       */
            Value |= (SartData[NetID][NetAddr] &  MASK_PTYERR) ? 0x04 : 0;
            Value |= (SartData[NetID][NetAddr-1] &  MASK_PTYERR) ? 0x01 : 0;
            sprintf(IOParseBuff+strlen(IOParseBuff),"%01x",Value);
         }
         RetCode = 0;
      }
      else {
         CmdShowMsg("*** Cannot use address or bit ID with this function\n");
      }
      break;
   }

Done:

   if (!RetCode) {
      Nputstr(IOParseBuff);                        /* show valid results     */
   }

   return RetCode;
}





/*---------------------------------------------------------------------------*/
/* Send the synchronizing message to get both SART nets lined up             */
/* The first bit is a zero and we make sure the sense input tracks both      */
/*  low and high voltage outputs.                                            */

IOSyncSarts() {

asm {

   CLR   INT0

*--- pull the line low and verify that the sense input tracks it down

   SETB  A_DIS_HIGH                    ; disable high drivers
   SETB  B_DIS_HIGH

   SETB  A_DIS_MID                     ; disable middle drivers
   SETB  B_DIS_MID

   SETB  A_EN_LOW                      ; enable low drivers
   SETB  B_EN_LOW

   Delay SartDly                       ; allow settling

   MOV   C,A_SENSE                     ; pick up line data bit
***CPL   C                             ; error if input low = line high
   MOV   FlagNetAFail,C                ; error if line high

   MOV   C,B_SENSE                     ; pick up line data bit
***CPL   C                             ; error if input low = line high
   MOV   FlagNetBFail,C                ; error if line high

   CLR   A_EN_LOW                      ; disable low drivers
   CLR   B_EN_LOW

*--- pull the line high and verify that the sense input tracks that, too
*    we cannot actually verify a high voltage, but this is probably OK

   CLR   A_DIS_HIGH                    ; enable high drivers
   CLR   B_DIS_HIGH

   Delay SartDly                       ; allow settling

   MOV   C,A_SENSE                     ; pick up line data bit
   CPL   C                             ; error if input low = line low
   ORL   C,FlagNetAFail
   MOV   FlagNetAFail,C

   MOV   C,B_SENSE                     ; pick up line data bit
   CPL   C                             ; error if input low = line low
   ORL   C,FlagNetBFail
   MOV   FlagNetBFail,C


}

/*--- send eight ONE bits                                                    */

asm {
   MOV   B,#8                          ; set up loop counter

Sync1Top

   SETB  A_DIS_HIGH                    ; high drivers off
   SETB  B_DIS_HIGH

   MOV   C,FlagNetAFail                ; turn on middle drivers
   MOV   A_DIS_MID,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_MID,C

   Delay SartDly                       ; wait for settling

   SETB  A_DIS_MID                     ; middle drivers off
   SETB  B_DIS_MID

   MOV   C,FlagNetAFail                ; turn on high drivers
   MOV   A_DIS_HIGH,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_HIGH,C

   Delay SartDly                       ; wait for settling


   DJNZ  B,Sync1Top

}


/*--- and the final ZERO bit                                                 */

asm {

   SETB  A_DIS_HIGH                    ; high drivers off
   SETB  B_DIS_HIGH

   MOV   C,FlagNetAFail                ; turn on low drivers
   CPL   C
   MOV   A_EN_LOW,C
   MOV   C,FlagNetBFail
   CPL   C
   MOV   B_EN_LOW,C

   Delay SartDly                       ; wait for settling

   CLR   A_EN_LOW                      ; low drivers off
   CLR   B_EN_LOW

   MOV   C,FlagNetAFail                ; turn on high drivers
   MOV   A_DIS_HIGH,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_HIGH,C

   Delay SartDly                       ; wait for settling


   SETB  INT0

}

/*--- set error indicator if either net is broken                            */

asm {

   CLR   A
   MOV   C,FlagNetAFail
   MOV   A.0,C
   MOV   C,FlagNetBFail
   MOV   A.1,C
   MOV   TestBits,A
}

   switch (TestBits) {
   case 1 :
      HBBits = HB_BADNETA;
      break;
   case 2 :
      HBBits = HB_BADNETB;
      break;
   case 3 :
      HBBits = HB_BADNETS;
      break;
   }

   return;

}

/*---------------------------------------------------------------------------*/
/* Send current SART address out to the networks                             */
/* Presumes the address is valid and the network has been synchronized       */
/* Both networks see the same address bits followed by even parity           */

IOSendAddr() {

asm {
   MOV   A,SartAddr                    ; pick up current address
   MOV   B,#5                          ; there are but five bits...

   CLR   INT1

AddrATop

   SETB  A_DIS_HIGH                    ; high drivers off
   SETB  B_DIS_HIGH

   RRC   A                             ; pick up low address bit in C
   JC    AddrA1                        ; which flavor of bit to send?

   MOV   C,FlagNetAFail                ; C clear means send 0 bit
   CPL   C
   MOV   A_EN_LOW,C
   MOV   C,FlagNetBFail
   CPL   C
   MOV   B_EN_LOW,C

   Delay SartDly                       ; wait for settling

   CLR   A_EN_LOW                      ; low drivers off
   CLR   B_EN_LOW

   SJMP  AddrA2

AddrA1 EQU *                           ; C set means send 1 bit

   MOV   C,FlagNetAFail                ; turn on middle drivers
   MOV   A_DIS_MID,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_MID,C

   Delay SartDly                       ; wait for settling

   SETB  A_DIS_MID                     ; middle drivers off
   SETB  B_DIS_MID

AddrA2 EQU *                           ; send out HIGH part of bit

   MOV   C,FlagNetAFail                ; turn on high drivers
   MOV   A_DIS_HIGH,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_HIGH,C

   Delay SartDly                       ; wait for settling


   DJNZ  B,AddrATop

}

/*--- send out even parity for the address bits                              */

asm {

   SETB  A_DIS_HIGH                    ; high drivers off
   SETB  B_DIS_HIGH

   MOV   A,SartAddr                    ; pick up address
   JB    EPTY,AddrAP1                  ; parity set if A has odd count

   MOV   C,FlagNetAFail                ; send 0 bit
   CPL   C
   MOV   A_EN_LOW,C
   MOV   C,FlagNetBFail
   CPL   C
   MOV   B_EN_LOW,C

   Delay SartDly                       ; wait for settling

   CLR   A_EN_LOW                      ; low drivers off
   CLR   B_EN_LOW

   SJMP  AddrAP2

AddrAP1  EQU *                         ;  send 1 bit

   MOV   C,FlagNetAFail                ; turn on middle drivers
   MOV   A_DIS_MID,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_MID,C

   Delay SartDly                       ; wait for settling

   SETB  A_DIS_MID                     ; middle drivers off
   SETB  B_DIS_MID

AddrAP2  EQU *                         ; send out HIGH part of bit

   MOV   C,FlagNetAFail                ; turn on high drivers
   MOV   A_DIS_HIGH,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_HIGH,C

   Delay SartDly                       ; wait for settling


   SETB  INT1

}

   return;

}




/*---------------------------------------------------------------------------*/
/* Send out a ZERO bit on both SART networks                                 */
/* This leaves the outputs enabled at a high level                           */

IOSendZero() {

asm {

   SETB  A_DIS_HIGH                    ; high drivers off
   SETB  B_DIS_HIGH

   MOV   C,FlagNetAFail
   CPL   C
   MOV   A_EN_LOW,C
   MOV   C,FlagNetBFail
   CPL   C
   MOV   B_EN_LOW,C

   Delay SartDly                       ; wait for settling

   CLR   A_EN_LOW                      ; low drivers off
   CLR   B_EN_LOW

   MOV   C,FlagNetAFail                ; turn on high drivers
   MOV   A_DIS_HIGH,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_HIGH,C

   Delay SartDly                       ; wait for settling


}

}

/*---------------------------------------------------------------------------*/
/* Send out a ONE bit on both SART networks                                  */
/* This leaves the outputs enabled at a high level                           */

IOSendOne() {

asm {

   SETB  A_DIS_HIGH                    ; high drivers off
   SETB  B_DIS_HIGH

   MOV   C,FlagNetAFail                ; middle drivers on
   MOV   A_DIS_MID,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_MID,C

   Delay SartDly                       ; wait for settling

   SETB  A_DIS_MID                     ; middle drivers off
   SETB  B_DIS_MID

   MOV   C,FlagNetAFail                ; turn on high drivers
   MOV   A_DIS_HIGH,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_HIGH,C

   Delay SartDly                       ; wait for settling


}

}

/*---------------------------------------------------------------------------*/
/* Poll a pair of SARTs and return the responses                             */
/* This gets called once per pass through the main loop                      */
/* We read the current state, then write the output data from the array      */
/* This clears any parity error and refreshes the output drivers             */

IOPollSarts() {

BYTE TempData;

/*--- step SART address and resynchronize if we are starting over again      */

   if (SartAddr >= (MAX_SARTS-1)) {
      SartAddr = 0;
      IOSyncSarts();
   }
   else {
      ++SartAddr;
   }

/*--- set up the SART address and send a zero to flag a readback             */

   IOSendAddr();
   IOSendZero();


/*--- pick up data and parity bits from SARTs                                */

asm {

   MOV   R0,#3                         ; number of bits to collect
   CLR   A                             ; set up data repositories
   MOV   B,A

PollDI1

   SETB  A_DIS_HIGH                    ; high drivers off
   SETB  B_DIS_HIGH

   MOV   C,FlagNetAFail                ; turn on middle drivers
   MOV   A_DIS_MID,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_MID,C

   Delay SartDly                       ; wait for settling

   MOV   C,A_SENSE                     ; pick up bit from Net A
***CPL   C                             ; flip polarity
   RLC   A
   MOV   C,B_SENSE                     ; pick up bit from Net B
***CPL   C                             ; flip polarity
   XCH   A,B
   RLC   A
   XCH   A,B

   SETB  A_DIS_MID                     ; middle drivers off
   SETB  B_DIS_MID

   MOV   C,FlagNetAFail                ; turn on high drivers
   MOV   A_DIS_HIGH,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_HIGH,C

   Delay SartDly                       ; wait for settling


   DJNZ  R0,PollDI1                    ; repeat for data and parity

*--- convert data+parity into data bits with parity error flag
*    the data is backwards with the parity in the low-order bit
*    a lookup table is quick and easy...
*    stash the results in the status table for later reference

   MOV   DPTR,#DInTab            ; point to translation table

   MOVC  A,[A+DPTR]              ; translate Net A bits
   MOV   R0,A                    ; save for later
   MOV   A,B                     ; repeat for Net B bits
   MOVC  A,[A+DPTR]
   MOV   R1,A

   MOV   DPTR,#SartData          ; aim at status array
   MOV   A,SartAddr              ; offset by current address
   ADD   A,DPL
   MOV   DPL,A
   MOV   A,DPH
   ADDC  A,#0
   MOV   DPH,A

   MOVX  A,[DPTR]                ; pick up existing Net A bits
   ANL   A,#~(MASK_PTYERR|MASK_IN1|MASK_IN0)       ; discard old bits
   ORL   A,R0                    ; combine with new bits
   MOVX  [DPTR],A                ; return to the array

   MOV   A,#MAX_SDATA            ; move pointer to Net B bits
   ADD   A,DPL
   MOV   DPL,A
   MOV   A,DPH
   ADDC  A,#0
   MOV   DPH,A

   MOVX  A,[DPTR]                ; pick up existing Net B bits
   ANL   A,#~(MASK_PTYERR|MASK_IN1|MASK_IN0)       ; discard old bits
   ORL   A,R1                    ; combine with new bits
   MOVX  [DPTR],A                ; return to the array

}

/*--- extract output data from the arrays and create the parity bits         */
/*    pass the data to the assembler code in TestBits for simplicity         */

   TempData = (SartData[0][SartAddr] & (MASK_OUT1 | MASK_OUT0)) >> OUT_SHIFT;
   TestBits = DOutTab[TempData];

   TempData = (SartData[1][SartAddr] & (MASK_OUT1 | MASK_OUT0)) >> OUT_SHIFT;
   TestBits |= DOutTab[TempData] << 4;

/*--- set up the SART address again and send a one bit to flag a write       */

   IOSendAddr();
   IOSendOne();

/*--- now send the data bits                                                 */
/*    for once, each network gets different bit values...                    */

asm {

   MOV   B,#3                          ; we send three pairs of bits
   MOV   A,TestBits                    ; picked out of this byte

OutLoop  EQU   *

   SETB  A_DIS_HIGH                    ; high drivers off
   SETB  B_DIS_HIGH

   MOV   C,FlagNetAFail                ; set up net fail bit for drivers
   JB    A.0,DOA1                      ; test Net A bit
   CPL   C                             ;  ... 0, so enable low drivers
   MOV   A_EN_LOW,C
   SJMP  DOutB

DOA1  EQU   *
   MOV   A_DIS_MID,C                   ;  ... 1, so enable middle drivers

DOutB EQU   *
   MOV   C,FlagNetBFail                ; set up other net fail bit

   JB    A.4,DOB1                      ; test Net B bit
   CPL   C                             ;  ... 0, so enable low drivers
   MOV   B_EN_LOW,C
   SJMP  DOutW

DOB1  EQU   *
   MOV   B_DIS_MID,C                   ;  ... 1, so enable middle drivers

DOutW EQU   *

   Delay SartDly                       ; wait for settling

   CLR   A_EN_LOW                      ; low drivers off
   CLR   B_EN_LOW
   SETB  A_DIS_MID                     ; middle drivers off
   SETB  B_DIS_MID

   MOV   C,FlagNetAFail                ; turn on high drivers
   MOV   A_DIS_HIGH,C
   MOV   C,FlagNetBFail
   MOV   B_DIS_HIGH,C

   Delay SartDly                       ; wait for settling


   RR    A                             ; slap bits around for next pass

   DJNZ  B,OutLoop                     ; and repeat...

}

/*--- indicated if we have done the last SART                                */
/*    we do not adjust anything here, so SartAddr indicates finished pass    */

   if (SartAddr >= (MAX_SARTS-1)) {                /* catch bad values...    */
      SETBIT(FlagEndScan);
   }

   return;

}


/*---------------------------------------------------------------------------*/
/* Exercise S-ART modules to help get things running                         */
/* This gets called every time the main loop finishes a complete scan        */
/*  therefore, the calling rate depends on the SART data rate...             */

IOTestSarts() {
BYTE TestAddr;

   ++SartTestPass;

   switch (SartTestMode) {
   case 0 :                                        /* do nothing             */
      break;
   case 1 :                                        /* identify each address  */
      if (SartTestPass >= MAX_SARTS) {
         SartTestPass = 0;
      }
      for (TestAddr=0; TestAddr < MAX_SARTS; TestAddr++) {
         if (TestAddr == SartTestPass) {
            SartData[0][TestAddr] |= (MASK_OUT0 | MASK_OUT1);
            SartData[1][TestAddr] |= (MASK_OUT0 | MASK_OUT1);
         }
         else {
            SartData[0][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
            SartData[1][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
         }
      }
      break;
   case 2 :                                        /* side-to-side blink     */
      for (TestAddr=0; TestAddr < MAX_SARTS; TestAddr++) {
         switch (SartTestPass & 0x03) {
         case 0 :
            SartData[0][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
            SartData[1][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
            break;
         case 1 :
            SartData[0][TestAddr] |= MASK_OUT0;
            SartData[0][TestAddr] &= ~MASK_OUT1;
            SartData[1][TestAddr] |= MASK_OUT0;
            SartData[1][TestAddr] &= ~MASK_OUT1;
            break;
         case 2 :
            SartData[0][TestAddr] &= ~MASK_OUT0;
            SartData[0][TestAddr] |= MASK_OUT1;
            SartData[1][TestAddr] &= ~MASK_OUT0;
            SartData[1][TestAddr] |= MASK_OUT1;
            break;
         case 3 :
            SartData[0][TestAddr] |= (MASK_OUT0 | MASK_OUT1);
            SartData[1][TestAddr] |= (MASK_OUT0 | MASK_OUT1);
            break;
         }
      }
      break;
   case 3 :                                        /* connect INx to OUTx    */
      for (TestAddr=0; TestAddr < MAX_SARTS; TestAddr++) {
         if (0 == (SartData[0][TestAddr] & MASK_PTYERR)) {
            SartData[0][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
            SartData[0][TestAddr] |=
               (SartData[0][TestAddr] & MASK_IN0) ? MASK_OUT0 : 0;
            SartData[0][TestAddr] |=
               (SartData[0][TestAddr] & MASK_IN1) ? MASK_OUT1 : 0;
         }
         if (0 == (SartData[1][TestAddr] & MASK_PTYERR)) {
            SartData[1][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
            SartData[1][TestAddr] |=
               (SartData[1][TestAddr] & MASK_IN0) ? MASK_OUT0 : 0;
            SartData[1][TestAddr] |=
               (SartData[1][TestAddr] & MASK_IN1) ? MASK_OUT1 : 0;
         }
      }
      break;
   case 4 :
      if (SartTestPass >= MAX_SARTS) {
         SartTestPass = 0;
      }
      for (TestAddr=0; TestAddr < MAX_SARTS; TestAddr++) {
         if (TestAddr == SartTestPass) {
            SartData[0][TestAddr] |= (MASK_OUT0 | MASK_OUT1);
         }
         else {
            SartData[0][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
         }
      }
      break;
   case 5 :
      if (SartTestPass >= MAX_SARTS) {
         SartTestPass = 0;
      }
      for (TestAddr=0; TestAddr < MAX_SARTS; TestAddr++) {
         if (TestAddr == SartTestPass) {
            SartData[1][TestAddr] |= (MASK_OUT0 | MASK_OUT1);
         }
         else {
            SartData[1][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
         }
      }
      break;
   case 6 :                                        /* side-to-side blink     */
      for (TestAddr=0; TestAddr < MAX_SARTS; TestAddr++) {
         switch (SartTestPass & 0x03) {
         case 0 :
            SartData[0][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
            break;
         case 1 :
            SartData[0][TestAddr] |= MASK_OUT0;
            SartData[0][TestAddr] &= ~MASK_OUT1;
            break;
         case 2 :
            SartData[0][TestAddr] &= ~MASK_OUT0;
            SartData[0][TestAddr] |= MASK_OUT1;
            break;
         case 3 :
            SartData[0][TestAddr] |= (MASK_OUT0 | MASK_OUT1);
            break;
         }
      }
      break;
   case 7 :                                        /* side-to-side blink     */
      for (TestAddr=0; TestAddr < MAX_SARTS; TestAddr++) {
         switch (SartTestPass & 0x03) {
         case 0 :
            SartData[1][TestAddr] &= ~(MASK_OUT0 | MASK_OUT1);
            break;
         case 1 :
            SartData[1][TestAddr] |= MASK_OUT0;
            SartData[1][TestAddr] &= ~MASK_OUT1;
            break;
         case 2 :
            SartData[1][TestAddr] &= ~MASK_OUT0;
            SartData[1][TestAddr] |= MASK_OUT1;
            break;
         case 3 :
            SartData[1][TestAddr] |= (MASK_OUT0 | MASK_OUT1);
            break;
         }
      }
      break;
   default :
      CmdShowMsg("*** Invalid test number\n");
      SartTestMode = 0;
   }


}

