/* CONSOLE.C -- console character I/O                                        */


/*---------------------------------------------------------------------------*/
/* Send character to console with LF to CR/LF translation                    */
/* If this is to the video display we turn off the video until a NL or EOT   */
/* We implement a tiny C-style escape sequence decoder here, too             */

/*--- the ?putch entry point has the character in A                          */
/*    this accounts for nearly half of the overall system stack use!         */

asm {
?putch
      PUSH  B
      PUSH  DPL
      PUSH  DPH
      PUSH  ?R0
      PUSH  ?R1
      PUSH  ?R2
      PUSH  ?R3
      PUSH  ?R4
      PUSH  ?R5
      PUSH  ?R6
      PUSH  ?R7
      PUSH  A
      PUSH  A
      LCALL putch
      POP   A
      POP   A
      POP   ?R7
      POP   ?R6
      POP   ?R5
      POP   ?R4
      POP   ?R3
      POP   ?R2
      POP   ?R1
      POP   ?R0
      POP   DPH
      POP   DPL
      POP   B
      RET
}

putch(TransChar)
BYTE TransChar;
{


asm {
      MOV   R0,#-5                                 /* aim R0 at TransChar    */
      LCALL ?auto0

/*--- no escape sequence active now, so                                      */
/*      if we find a backslash we become active                              */
/*      otherwise, just show the char                                        */

      MOV   A,ConEscState
      CJNE  A,#CESC_IDLE,pch_notidle

      CJNE  [R0],#'\\',pch_go_near

      MOV   ConEscState,#CESC_ACTIVE
      MOV   [R0],#0
pch_go_near
      AJMP  pch_go

pch_notidle

/*--- previous char was a backslash, so decode current char                  */

      MOV   ConEscState,#CESC_IDLE                 /* default next state     */

      CJNE  A,#CESC_ACTIVE,pch_notactive

      MOV   A,[R0]                                 /* convert to lower       */
      PUSH  A
      CLR   A
      PUSH  A
      LCALL tolower
      DEC   SP
      DEC   SP

      MOV   R0,#-5                                 /* aim R0 back at param   */
      LCALL ?auto0
      MOV   [R0],A

      CJNE  A,#'b',pch_nb              /* bell char, which we ignore         */
      MOV   [R0],#0
      AJMP  pch_go
pch_nb

      CJNE  A,#'c',pch_nc              /* control char, wait for next char   */
      MOV   ConEscState,#CESC_CTRLCHAR
      MOV   [R0],#0
      AJMP  pch_go
pch_nc

      CJNE  A,#'e',pch_ne              /* escape, substitute ASCII ESC       */
      MOV   [R0],#ESC
      AJMP  pch_go
pch_ne

      CJNE  A,#'f',pch_nf              /* form feed                          */
      MOV   [R0],#FORMFEED
      AJMP  pch_go
pch_nf

      CJNE  A,#'n',pch_nn              /* newline, which triggers CR         */
      MOV   [R0],#NL
      AJMP  pch_go
pch_nn

      CJNE  A,#'r',pch_nr              /* return                             */
      MOV   [R0],#CR
      AJMP  pch_go
pch_nr

      CJNE  A,#'t',pch_nt              /* tab                                */
      MOV   [R0],#TAB
      AJMP  pch_go
pch_nt

      CJNE  A,#'x',pch_nx              /* hex sequence, wait for hexits      */
      MOV   ConEscState,#CESC_ONEHEX
      MOV   [R0],#0
      AJMP  pch_go
pch_nx

      CJNE  A,#'\\',pch_nbslash
      MOV   [R0],#'\\'
      AJMP  pch_go
pch_nbslash

      AJMP  pch_go

pch_notactive

/*--- create control char from current char                                  */

      CJNE  A,#CESC_CTRLCHAR,pch_notctrl
      MOV   A,[R0]
      ANL   A,#$1F
      MOV   [R0],A
      AJMP  pch_go

pch_notctrl

/*--- convert and save first hexit                                           */

      CJNE  A,#CESC_ONEHEX,pch_not1hex
      MOV   A,[R0]
      PUSH  A
      CLR   A
      PUSH  A
      LCALL CvtHexit
      DEC   SP
      DEC   SP
      SWAP  A                                      /* hexit to high nybble   */
      MOV   ConEscValue,A                          /* save for next pass     */
      MOV   ConEscState,#CESC_TWOHEX
      MOV   R0,#-5                                 /* aim R0 back at param   */
      LCALL ?auto0
      MOV   [R0],#0
      AJMP  pch_go

pch_not1hex

/*--- convert second hexit, combine, and display                             */

      CJNE  A,#CESC_TWOHEX,pch_not2hex
      MOV   A,[R0]
      PUSH  A
      CLR   A
      PUSH  A
      LCALL CvtHexit
      DEC   SP
      DEC   SP
      ORL   A,ConEscValue                          /* combine hexits         */
      MOV   R0,#-5                                 /* aim R0 back at param   */
      LCALL ?auto0
      MOV   [R0],A
      AJMP  pch_go

pch_not2hex

/*--- we have an invalid state, which we force to idle and continue          */

      MOV   ConEscState,#CESC_IDLE

pch_go
}

   if (TransChar) {
      if (NL == TransChar) {
         putch(CR);
      }

      TESTBIT(TestBits,B_ConVideo);
      if (TestBits) {
         if ((NL == TransChar) || (EOT == TransChar)) {
            SETBIT(B_VidEnable);       /* allow enable after retrace         */
         }
         else {
            CLRBIT(B_VidEnable);       /* force disable after retrace        */
            CLRBIT(IO_VIDENABLE);      /* disable immediately                */
         }
         if (EOT != TransChar) {
            ANSIPutChar(TransChar);
         }
      }
      else {
         SerPutChar(TransChar);
      }
   }

}



/*---------------------------------------------------------------------------*/
/* Get character from console                                                */
/* Discards LFs and converts CRs into NLs                                    */

int getch() {
BYTE RetValue;

   do {
      RetValue = SerGetChar();
   } while (LF == RetValue);

   if (CR == RetValue) {
      RetValue = NL;
   }

   return RetValue;


}



/*---------------------------------------------------------------------------*/
/* Send string to console from code space (this =is= tiny model code!)       */
/*  putstr(char *pString);                                                   */
/*  The ?putstr entry point has *pString in DPTR                             */

putstr(pString)
char *pString;
{

 asm {
   MOV   R0,#-5            ; get string pointer from stack
   LCALL ?auto0
   MOV   DPL,[R0]          ; low byte
   INC   R0
   MOV   DPH,[R0]
?putstr
   CLR   A
   MOVC  A,[A+DPTR]        ; fetch character from CODE space
   JZ    pstr1             ; if zero, done
   PUSH  DPH               ; save pointer
   PUSH  DPL
   PUSH  A                 ; set up parameter
   PUSH  A
   LCALL putch             ; send it out
   POP   A                 ; discard parameter
   POP   A
   POP   DPL               ; restore pointer
   POP   DPH
   INC   DPTR
   SJMP  ?putstr           ; continue...

pstr1 EQU *
   RET                     ; *** bypass the normal return
 }

   pString = 0;            /* dummy code to reference pString...             */

}




/*---------------------------------------------------------------------------*/
/* Convert a character into the corresponding hex value                      */
/* you get evil results if the char is something other than a valid hexit    */

CvtHexit(CharCode)
BYTE CharCode;
{

asm {

   MOV   R0,#-5
   LCALL ?auto0
   MOV   A,[R0]

   CJNE  A,#'a',CH_1a         * C set if A is digit or uppercase
CH_1a
   JC    CH_1
   ADD   A,#-$20              * convert lowercase to upper
CH_1

   CJNE  A,#$3A,CH_2a         * C set if digit
CH_2a
   JC    CH_2
   ADD   A,#-7                * convert A-F to 3A-3F
CH_2

   ADD   A,#-'0'              * convert to binary

   MOV   [R0],A

}

   return CharCode;

}
