/* Home Control System II                                                    */
/* Video Terminal                                                            */
/* Copyright 1995 Circuit Cellar Inc                                         */

/* Compiled with Micro-C using special HCS Tiny model                        */
/* No external RAM... it's all a video buffer                                */
/*   even though we do use some of it for data buffering...                  */

#include <8051reg.h>
#include <8051io.h>

#include "e:\mc\cust51\8051base.h"
#include "e:\mc\cust51\8051sfb.h"

/*---------------------------------------------------------------------------*/
/* Constants                                                                 */

#include "ansi.h"
#include "chargen.h"
#include "command.h"
#include "console.h"
#include "serial.h"
#include "vidlink.h"
#include "video.h"

/*---------------------------------------------------------------------------*/
/* Internal RAM variables below the bit RAM                                  */

/*--- these start at 0008 as defined in 8051rlpt.asm                         */


#include "chargen.d1"
#include "console.d1"
#include "network.d1"
#include "serial.d1"
#include "video.d1"

/*-- define some padding to force the last variable here to address 0x1F     */

NEAR BYTE Padding[0x1F - CG_D1 - CON_D1 - NET_D1 - SER_D1 - VID_D1 - 8];
NEAR BYTE TestBits;                    /* 1F global temp for TESTBIT macro   */

/*---------------------------------------------------------------------------*/
/* Bit variables starting at 0x20                                            */
/* We force a covering byte at the right address by a sleazy manual trick    */
/*  ... if the padding shown above isn't right, we're dead...                */

NEAR BYTE BitFlags20;                  /* must be at 0x20                    */

#define B_VidEnable     $20.0          /* 1 = enable video when appropriate  */
#define B_CGDoubleSize  $20.1          /* 1 = use BIG characters             */
#define B_ConVideo      $20.2          /* 1 = video, 0 = serial console I/O  */
#define B_NetUseCS      $20.3          /* 1 = verify & compute checksums     */
#define B_LineWrap      $20.4          /* 1 = wrap at line end               */
#define B_NetInHeader   $20.5          /* 1 = processing net message header  */
#define B_Field2        $20.6          /* 1 = Field 2, 0 = Field 1           */

NEAR BYTE BitFlags21;                  /* must be at 0x21                    */

#define B_SerOverrun    $21.0          /* set when rec overruns input FIFO   */
#define B_SerEcho       $21.1          /* 1 = echo chars directly from rec   */
#define B_SerUseTE      $21.2          /* 1 = use transmitter enable output  */
#define B_SerTransIdle  $21.3          /* set when transmitter becomes idle  */


/*---------------------------------------------------------------------------*/
/* Variables in Internal RAM and constants in EPROM                          */
/* We have no external RAM, which complicates a lot of things...             */

NEAR BYTE SwitchCopy;                  /* holds copy of switches after reset */

#include "ansi.d2"
#include "command.d2"
#include "network.d2"
#include "serial.d2"
#include "video.d2"


/*---------------------------------------------------------------------------*/
/* Pick up the rest of the code                                              */

#include "ansi.c"
#include "chargen.c"
#include "command.c"
#include "console.c"
#include "network.c"
#include "serial.c"
#include "video.c"
#include "vidsync.c"


/*---------------------------------------------------------------------------*/
/* The Main Event                                                            */
/* The initial message must fit on a 15x15 display... watch those margins!   */

char InitMsg[] = "HCSII Vid-Link  TV Interface\\e[K\n\
 Version 1.0b\\e[K\n\n\
Copyright 1995\\e[K\n Circuit Cellar\\e[K Incorporated\\e[K\n\n";


main() {
BYTE OldField;
BYTE CmdChar;

/*--- initialization routines                                                */

   BitFlags20 = 0;                     /* and all the bit flags              */
   BitFlags21 = 0;

 asm {
   MOV   R0,SP                         ; clear the stack for bounds check
   INC   R0
   CLR   A
?ilp
   MOV   [R0],A
   INC   R0
   CJNE  R0,#LOW(MAXSTACK+1),?ilp      ; can wrap from FF to 100 just fine

   MOV   DPTR,#SWITCHES                ; pick up switches for later use
   MOVX  A,[DPTR]
   CPL   A                             ; make ON = 1
   MOV   SwitchCopy,A

 }

   VidInit();

   SerInit();

/*   console                                                                 */

   SETBIT(B_ConVideo);                 /* send initial output to video       */
   ConEscState = CESC_IDLE;            /* and clear the escape state machine */
   ConEscValue = 0;

/*--- all the other initializations                                          */

   ANSIInit();                         /* set up video ANSI controls         */

   SETBIT(EA);                         /* interrupts are OK now              */

   CGenInit();                         /* set up character generator         */

   putstr(InitMsg);                    /* show intro on TV screen            */

   NetInit();                          /* set up network interface           */
   CmdInit();                          /* set up command decoder             */

   if (0 == CmdMode) {                 /* send intro to serial port, too     */
      CLRBIT(B_ConVideo);
      putch(FORMFEED);
      putstr(InitMsg);
      SETBIT(B_ConVideo);
   }

   OldField = VidFieldCtr;

/*--- wait for first complete line addressed to us                           */
/*    then clear the screen and enter the main loop                          */

   CmdShowPrompt();                    /* show prompt if in human mode       */
   CmdChar = CmdWaitCommand();

   CGenClear();

   while (TRUE) {

      CmdDecode(CmdChar);              /* decode the command                 */
      CmdShowPrompt();                 /* show prompt if in human mode       */
      CmdChar = CmdWaitCommand();      /* wait for the next one              */

   }

}

