/* Convert binary font file to ASM source code                               */

TabChar = '09'x
BitImages = ' ' || '08'x

parse upper arg InFileName BytesPerChar

if (InFileName = "") & (BytesPerChar = "")
then do
   say "syntax is bintoasm infile.BIN bytes-per-char"
   exit 1
end

if BytesPerChar = ""
then do
   say "Assuming 8 bytes per character"
   BytesPerChar = 8
end

if 0 = lastpos(".BIN",InFileName)
then do
   say "Assuming input extension..."
   InFileName = InFileName || ".BIN"
end

say "Input font file is" InFileName
FileSize = chars(InFileName)
say " size is" FileSize "bytes"

NumChars = FileSize % BytesPerChar
Ooops = FileSize // BytesPerChar

if 0 = Ooops
then do
   say " contains" NumChars "character cells"
end
else do
   say "*** File doesn't have a whole number of characters!"
   exit 1
end

OutFileName = doseditname(InFileName,"*.ASM")
say "Output assembler file is" OutFileName
call sysfiledelete(OutFileName)

/*--- set up the file header                                                 */

call lineout OutFileName,"* Font file       " InFileName
call lineout OutFileName,"* " format(NumChars,5) "chars"
call lineout OutFileName,"* " format(BytesPerChar,5) "bytes/char"
call lineout OutFileName,"* " format(FileSize,5) "total bytes"
call lineout OutFileName,"*"

do CharID=0 to NumChars-1
   call lineout OutFileName,"* Character" d2x(CharID,2)
   do ByteID = 0 to BytesPerChar-1
      ThisByte = Charin(InFileName)
      LineData = TabChar || "DB" || TabChar || "$" || c2x(ThisByte)
      LineData = LineData || TabChar || "* "
      ByteImage = Translate(x2b(c2x(ThisByte)),BitImages,'01')
      LineData = LineData || ByteImage
      call lineout OutFileName,LineData
   end
   call lineout OutFileName,"*"
end

