/* COMMAND.C -- command decoder                                              */
/* Also includes network address recognition & response code                 */





/*---------------------------------------------------------------------------*/
/* Initialize command decoder                                                */

CmdInit() {

   CSErrCount = 0;

   CmdCheckMode();               /* figure out manual or network mode        */

}


/*---------------------------------------------------------------------------*/
/* Wait for complete line addressed to us                                    */
/* The serial handlers discard lines to other nodes                          */

int CmdWaitCommand() {
int RetChar;

#ifdef xxx
   putstr("Waiting for line, state: ");
   putch('0' + SerState);
   putstr("\n");
#endif

   RetChar = getch();

#ifdef xxx
   putstr(" Got the line!\n");
#endif

   return RetChar;

}


/*---------------------------------------------------------------------------*/
/* Send out a prompt in plain interactive mode                               */
/* This bypasses the checksum buffer, for obvious reasons...                 */

CmdShowPrompt() {

   if (0 == CmdMode) {                 /* network modes?  0 = human...       */
      CLRBIT(B_ConVideo);              /* this goes to the serial port       */

      putstr(NetNames[NetNameSel]);
      putch('0'+NetNodeNum);
      putstr(" >");                    /* blank and prompt character         */
      putch(EOT);                      /* trigger the output transmission    */

      SETBIT(B_ConVideo);              /* back to the TV                     */
   }

}



/*---------------------------------------------------------------------------*/
/* Check for manual mode                                                     */
/* Sets up serial and network parameters to support command I/O modes        */

int CmdCheckMode() {

   if (SwitchCopy & 0x01) {            /* switch ON = manual mode            */
      CmdNetMode(0);                   /* yes, set up controls               */
   }
   else {
      CmdNetMode(NETMODE);             /* no, set up network mode            */
   }

   return;

}


/*---------------------------------------------------------------------------*/
/* Switch network mode ON or OFF depending on argument                       */

CmdNetMode(State)
int State;
{

   SerWaitOutDone();                   /* wait for serial port to finish...  */

   CmdMode = State;

   SETBIT(IO_TRANSDISABLE);            /* disable net transmitter            */

   switch (CmdMode) {
   default:                            /* oops...                            */
      CmdMode = 0;
   case 0 :                            /* interactive human mode             */
      CLRBIT(B_NetUseCS);                          /* no checksums, natch    */
      SETBIT(B_SerEcho);                           /* echo each character    */
      CLRBIT(B_SerUseTE);                          /* don't enable trans     */
      break;
   case 1 :                            /* network mode with no feedback      */
      SETBIT(B_NetUseCS);                          /* use checksums          */
      CLRBIT(B_SerEcho);                           /* don't echo chars       */
      SETBIT(B_SerUseTE);                          /* must enable/disable    */
      break;
   }

}


/*---------------------------------------------------------------------------*/
/* Decode the commands                                                       */
/* This must use all the inbound chars before sending any output             */
/*  ... we have only one Internal RAM FIFO that can go in only one direction */
/* We use tolower() because it is used elsewhere in the asm code             */

CmdDecode(CmdChar)
int CmdChar;
{

   while (' ' == CmdChar) {            /* strip leading blanks               */
      CmdChar = getch();
   }

   CLRBIT(B_ConVideo);                 /* default output to serial port      */

   switch (tolower(CmdChar)) {
   default :
   case 0 :                            /* ignore empy line                   */
      SerFlushInput();
      break;
   case 'n' :                          /* select net mode                    */
      if (SerCheckChar()) {
         CmdNetMode(getch() - '0');
         SerFlushInput();
      }
      else {
         if (0 == CmdMode) {
            putstr("Net mode is ");
            putch('0' + CmdMode);
            putch('\n');
         }
      }
      break;
   case 's' :                          /* send message to LCD panel          */
      if ('=' == getch()) {                        /* verify S= format       */
         SETBIT(B_ConVideo);
         while (SerCheckChar()) {
            putch(getch());
         }
         putch(EOT);                   /* make sure video is enabled         */
         CLRBIT(B_ConVideo);
      }
      else {
         SerFlushInput();
      }
      break;
   case '?' :
      if (0 == CmdMode) {
         SerFlushInput();
         putstr("Commands\n");
         putstr(" Nn    set net mode\n");
         putstr(" S=str show string\n");
      }
      break;
   }

   SETBIT(B_ConVideo);                 /* default output back to video       */

   return;

}


