/* Video routines for Vid-Link                                               */

#define TEST_ZEROFILLxxx               /* add xxx to display test pattern    */

/*---------------------------------------------------------------------------*/
/* Video initialization                                                      */
/* We set the hardware up as though we are in the middle of Field 2          */

VidInit(void) {

   VidStateID = VS_F2_END;             /* next state will end Field 2        */
   VidHandlerID = 2*VS_F2_END;         /* and this means a good startup      */
   VidFieldCtr = 0;                    /* Frame 0 Field 1 will be 01         */
   SETBIT(B_VidEnable);                /* allow display when ready           */

   PumpState = PUMP_IDLE;              /* no data movement yet               */
   PumpFIFOHead = PumpFIFOTail = PumpFIFO;         /* address, not index!    */
   PumpFIFOCtr  = 0;
   PumpLine = PUMP_RECLINE;


asm {

      SETB  IO_RAMDISABLE              * shut off the RAM during setup
      CLR   IO_VIDENABLE               * and disable the video

*--- set up Timer 0 so the video interrupts will eventually see it

      ANL   TCON,#%11001111            * flush Timer 0 enable & flag

      ANL   TMOD,#%11110000            * flush Timer 0 controls
      ORL   TMOD,#%00000001            * Timer 0 to Mode 0, 16-bit

      MOV   TH0,#HIGH -T0_ACTIVE       * delay for a goodly while
      MOV   TL0,#LOW  -T0_ACTIVE

      SETB  TR0                        * start it up with TF0 = 0

*--- get sync pulses running correctly

      MOV   DPTR,#I8254_CONTROL
      MOV   A,#%00010100               * ctr 0 lsb mode 1 binary
      MOVX  [DPTR],A                   *  controls sync period

      MOV   A,#%01010010               * ctr 1 lsb mode 1 binary
      MOVX  [DPTR],A                   *  controls sync pulse duration

      MOV   A,#%10011010               * ctr 0 lsb mode 1 binary
      MOVX  [DPTR],A                   *  controls blanking duration

      MOV   DPTR,#I8254_PERIOD         * set up sync period
      MOV   A,#PERIOD_NORMAL
      MOVX  [DPTR],A

      INC   DPTR                       * set up sync pulse width
      MOV   A,#SYNC_NORMAL
      MOVX  [DPTR],A

      INC   DPTR                       * set up blanking duration
      MOV   A,#BLANK_NORMAL
      MOVX  [DPTR],A

*--- fill visible part of video RAM with a test pattern
*    the last two bytes are not displayed correctly due to hardware limits

      CLR   IO_RAMDISABLE              * turn on the RAM outputs

      MOV   DPTR,#VIDEO_BUS+$FF        * aim at RAM, last dot
      MOV   P1,#MAX_ACTIVE_LINES-1     * start at last visible row
      MOV   R7,#0                      * lines have 256 bytes each

?VI_test
#ifdef TEST_ZEROFILL
      CLR   A                          * force zero background
#else
      MOV   A,P1                       * use address as data
      CLR   C
      SUBB  A,DPL
#endif
      MOVX  [DPTR],A                   * set it into the RAM
      DEC   DPL                        * repeat for whole line
      DJNZ  R7,?VI_test
      DEC   P1                         * and all lines until P1 wraps
      MOV   A,P1
      CJNE  A,$FF,?VI_test             * step to next line

*--- clear the rest to binary zeros
*     these are invisible and used for data storage
*     P1 has wrapped to FF so we need not reset it

      MOV   A,#MAX_ACTIVE_LINES-1      * compute line count (0-active lines)
      CPL   A
      MOV   B,A
      CLR   A
      MOV   R7,A                       * 256 bytes per line again
?VI_clear
      MOVX  [DPTR],A
      DEC   DPL                        * fill the line
      DJNZ  R7,?VI_clear
      DEC   P1                         * step to next line
      DJNZ  B,?VI_clear

*--- hack a dot near a trouble spot...

*     MOV   P1,#231
*     MOV   DPTR,#VIDEO_BUS+251
*     MOV   A,#$00
*     MOVX  [DPTR],A

*--- set up the interrupt hardware

      SETB  PX0                        * video is highest priority
      SETB  IT0                        * edge sensitive
      SETB  EX0                        * and enabled

}

}



/*---------------------------------------------------------------------------*/
/* Write a single data byte into the buffer as eight visible dots            */

#ifdef xxx

VidWriteDataByte(DataBits,DataAddr)
BYTE DataBits;
WORD DataAddr;
{

asm {
   MOV   C,B_VidEnable                 /* preserve visibility                */
   MOV   F0FLAG,C
   CLR   IO_VIDENABLE

   MOV   R0,#-7                        /* snag params from stack             */
   LCALL ?auto0                        /*  ... aim R0 at DataBits            */
   MOV   ?R1,[R0]                      /* DataBits                           */
   INC   R0
   INC   R0
   MOV   DPL,[R0]                      /* DataAddr low byte                  */
   INC   R0
   MOV   P1,[R0]                       /* DataAddr high byte                 */

   MOV   B,#8                          /* do eight bits at a crack           */
   MOV   DPH,#HIGH VIDEO_BUS           /* aim at video buffer                */

?VWB_1
   MOV   A,R1                          /* pick up the bits                   */
   RLC   A
   MOV   R1,A                          /* save bits for next pass            */
   CLR   A                             /* assume bit was off                 */
   SUBB  A,#0                          /*  0 - 0 - bit gives either 00 or FF */
   MOVX  [DPTR],A
   INC   DPL                           /* step to next buffer byte           */
   DJNZ  B,?VWB_1

   MOV   P1,VidActiveLine              /* restore currently active line      */
   MOV   C,F0FLAG                      /* restore visibility                 */
   MOV   B_VidEnable,C
   MOV   IO_VIDENABLE,C

   RET                           /**** bypass normal return                  */
}

   DataBits = DataAddr;                /* keep the compiler happy            */

}

#endif


